# mover2.rb (C) 2007 jim.foltz@gmail.com
#
# Use: Move Groups/Components around using arrows
#
require "sketchup"
#require "dev/debug"

class Sketchup::ComponentInstance
   def copy
      parent = self.parent
      cdef = self.definition
      ins = parent.entities.add_instance(cdef, self.transformation)
      ins.material = self.material
      return ins
   end
end
class Sketchup::Group
   alias :old_copy :copy
   def copy
      g = self.old_copy
      g.make_unique
      g.material = self.material
      return g
   end
   def moveto(pt)
      t = self.transformation.to_a
      t[12] = pt.x
      t[13] = pt.y
      t[14] = pt.z
      self.transformation = t
   end
end


class Others
   attr_accessor :ents
   def initialize ents=nil
      @ents=[]
      add ents
      @ents
   end
   def length
      @ents.length
   end
   def each
      @ents.each{|x| yield x}
   end
   def bounds
      re=Geom::BoundingBox.new
      @ents.each{|x| re.add x.bounds}
      re
   end
   def transformation
      Geom::Transformation.new bounds.center
      ##wikii~~ Geom::Transformation.new bounds.min
   end
   def transformation= tr
      transform! tr
   end

   def transform! t
      $ee.transform_entities t,@ents
   end
   def clear
      @ents=[]
   end
   def add ents
      @ents << ents
      @ents.flatten!
      @ents.uniq!
      @ents.compact!
      @ents.delete_if{|x| !x.is_a?(Sketchup::Drawingelement)}
      @ents
   end

end
##wikii~~ a=Others.new
##wikii~~ a.add $ss.to_a
##wikii~~ a.transform! [1,1,100]
##wikii

class Mover3 < UI::WebDialog
   AXES = { "y" => Y_AXIS, "z" => Z_AXIS, "x" => X_AXIS } if !defined?(AXES)
   UNITS = [1.0, 12.0, 25.4, 2.54, 0.0254] if !defined?(UNITS)

   def initialize(fn)
      super fn.capitalize, false, fn.capitalize, 200,200,200,390,true
      ##wikii~~ a=File.join(File.dirname(__FILE__),  "#{fn}.html")
      a=File.join($sut.path,  "plugins/mover3/#{fn}.html")
      puts a
      set_file File.join($sut.path,  "plugins/mover3/#{fn}.html")
      @keys = {"17" => false, "16" => false}
      @axis = AXES["z"]
      set_size(172,375)
      show {
         mv_amt = Sketchup.read_default("Mover3", "move_amt", "1000")
         mv_amt.gsub!(/ft/, "\\\\'")
         mv_amt.gsub!(/in/, '\"')
         script = %/o=document.getElementById("move_amt").value='#{mv_amt}';/
         self.execute_script(script)
         rot_amt = Sketchup.read_default("Mover3", "rot_amt", 90)
         script = %/o=document.getElementById("rot_amt").value='#{rot_amt}';/
         self.execute_script(script)
      }

      ##wikii~~ add_action_callback("keyDownHandler") do |d, a|
      ##wikii~~ #puts "keyDownHandler:#{a.inspect}"
      ##wikii~~ @keys[a] = true
      ##wikii~~ #p @keys
      ##wikii~~ end

      ##wikii~~ add_action_callback("keyUpHandler") do |d, a|
      ##wikii~~ # puts "keyUpHandler:#{a.inspect}"
      ##wikii~~ @keys[a] = false
      ##wikii~~ # p @keys
      ##wikii~~ end

      add_action_callback("move") do |d, params|
         #puts "move cb:#{params.inspect}"
         #p @keys
         if( @keys["16"] ) # Shift
            slide(d, params)
            return
         end
         if params == "o"
            move0
         else
            move(d, params)
         end
      end
      add_action_callback("rotate") do |d, a|
         #puts "rotate cb:#{a.inspect}"
         dir, ang = a.split(",")
         rotate(dir, ang.to_f)
      end
      add_action_callback("drop")  do |d, a|
         drop(d, a)
      end
      add_action_callback("align") do |d, a|
         #puts "align:#{a.inspect}"
      end
      add_action_callback("undo") do
         Sketchup.undo
      end
      add_action_callback("setAxis") do |d, a|
         # puts "setAxis:#{a.inspect}"
         @axis = AXES[a]
      end
      add_action_callback("top") {| d, a| Sketchup.send_action("viewTop:") }

      set_on_close {
         amt = self.get_element_value("move_amt")
         amt.gsub!(/'/, 'ft')
         amt.gsub!(/"/, 'in')
         Sketchup.write_default("Mover2", "move_amt", amt)
         amt = self.get_element_value("rot_amt")
         Sketchup.write_default("Mover2", "rot_amt", amt)
      }

   end # initialize


   def camera
      Sketchup.active_model.active_view.camera
   end

   def slide(d, a)
      #puts "slide:#{a.inspect}"
      e = Sketchup.active_model.selection[0]
      return unless e
      ec = e.bounds.center
      pt = nil
      case a
      when "e"
         v = X_AXIS.clone
         v.length = e.bounds.width / 2
      when "w"
         v = X_AXIS.reverse
         v.length = e.bounds.width / 2
      when "n"
         v = Y_AXIS.clone
         v.length = e.bounds.height / 2
      when "s"
         v = Y_AXIS.reverse
         v.length = e.bounds.height / 2
      end
      pt = ec + v
      raytest = Sketchup.active_model.raytest(pt, v)
      if raytest
         #Sketchup.active_model.entities.add_cpoint(raytest[0])
         #Sketchup.active_model.entities.add_cline(pt, v)
         v = raytest[0] - pt
         e.transform!(v)
      end

   end

   def moveto(e, pt)
      o = e.transformation.origin
      if pt.is_a?(Array)
         pt = Geom::Point3d.new(pt)
      end
      v = pt - o
      tr = Geom::Transformation.translation(v)
      e.transform!(tr)
   end

   def move0
      elms = Sketchup.active_model.selection.find_all {|el| el.typename == "Group" or el.typename == "ComponentInstance"}
      else_ent=Sketchup.active_model.selection.to_a-elms
      select=Others.new
      select.add else_ent
      elms << select if select.length>0
      return unless elms.size > 0
      #e.transformation = Geom::Transformation.new
      elms.each{|e|
         moveto(e, [0,0,0])
      }
   end
   def move(d, params)
      #puts "move:#{params.inspect}"
      elms = Sketchup.active_model.selection.find_all {|el| el.typename == "Group" or el.typename == "ComponentInstance"}
      else_ent=Sketchup.active_model.selection.to_a-elms
      select=Others.new
      select.add else_ent
      elms << select if select.length>0
      return unless elms.size > 0
      #unless( e.typename == "Group" or e.typename == "ComponentInstance")
      #UI.messagebox("Please select a group or component.")
      #break
      #end
      Sketchup.active_model.start_operation("move")
      amt = d.get_element_value("move_amt").to_f
      return if amt==0
      #puts "amt is: #{amt.inspect}"
      unit_options = Sketchup.active_model.options["UnitsOptions"]
      units_index = unit_options["LengthUnit"]
      #puts "units are: #{units_index.inspect}"
      # 0 = inches
      # 1 = feet
      # 2 = millimeters
      # 3 = centimeters
      # 4 = meters
      amt = amt / UNITS[units_index]
      #puts "amt is: #{amt.inspect}"
      elms.each do |e|
         bb = e.bounds
         x = y = z = 0
         case params
         when "n"
            y = amt
            y = bb.height if amt == 0
         when "ne"
            x = y = amt
            if amt == 0
               x = bb.width
               y = bb.height
            end
         when "e"
            x = amt
            x = bb.width if amt == 0.0
         when "se"
            x = amt
            y = -amt
            if amt == 0
               x = bb.width
               y = -bb.height
            end
         when "s"
            y = -amt
            y = -bb.height if amt == 0.0
         when "sw"
            y = -amt
            x = -amt
            if amt == 0
               y = -bb.height
               x = -bb.width
            end

         when "w"
            x = -amt
            x = -bb.width if amt == 0.0
         when "nw"
            y = amt
            x = -amt
            if amt == 0
               y = bb.height
               x = -bb.width
            end
         when "u"
            z = amt
            z = bb.depth if amt == 0.0
         when "d"
            z = -amt
            z = -bb.depth if amt == 0.0
         end # case
         pt = [x, y, z]

         if !["u" "d"].include?(params) and shift?
            v=camera.direction
            v.z=0
            v=camera.up  if v.length==0
            t=Geom::Transformation.rotation [0,0,0], [0,0,1], v.angle_between([0,1,0])
            pt.transform! t
         end


pt=Geom::Vector3d.new(pt).transform! $mm.edit_transform
p pt
p $mm.edit_transform.zaxis
p pt
        tr = Geom::Transformation.new pt
         #if @cb == "false" # move, no copy
         if ctrl? #@keys["17"] == true # move, no copy
            if e.is_a?(Others)
               e=$ee.add_group(e.ents)
               e=e.to_component
               $ss.add e
            end
            e1 = e.copy
            ##wikii~~ e.explode
            #e.transform! tr if e.respond_to?("transform!")
         end
         if false # camera follow preview
            v = Sketchup.active_model.active_view
            c = v.camera
            c.set c.eye.transform(tr), e.transformation.origin, c.up
         end
         e.transform!(tr) if e.respond_to?("transform!")
         #e.moveto(pt)
      end # elms.each
      Sketchup.active_model.commit_operation
   end # move call_back

   def _move(o, v)
      tr = Geom::Transformation.translation(v)
      o.move!(tr) if o.respond_to?("move!")
   end # _move

   def rotate(dir, a)
      #angle = d.get_element_value("angle").to_i
      ##wikii~~ sel = Sketchup.active_model.selection[0]
      ##wikii~~ return unless sel

      return if Sketchup.active_model.selection.length==0
      if dir == "r" # random rotate all selected
         Sketchup.active_model.start_operation("Random Rotate")
         elms = Sketchup.active_model.selection.find_all {|el| el.typename == "Group" or el.typename == "ComponentInstance"}
         else_ent=Sketchup.active_model.selection.to_a-elms
         select=Others.new
         select.add else_ent
         elms << select if select.length>0
         return unless elms.size > 0

         elms.each do |s|
            next unless s.respond_to?("transformation")
            #r = 360 * rand
            r = rand(a*2.0+1.0)-a
            #print "#{r} "
            #tr = Geom::Transformation.rotation(s.transformation.origin, [0, 0, 1], r.degrees)
            tr = Geom::Transformation.rotation(s.transformation.origin, @axis, r.degrees)
            s.transform!(tr)
         end
         Sketchup.active_model.commit_operation
         return
      end

      if dir == "z" # reset rotations
         Sketchup.active_model.start_operation("Reset Rotation")
         elms = Sketchup.active_model.selection.find_all {|el| el.typename == "Group" or el.typename == "ComponentInstance"}
         else_ent=Sketchup.active_model.selection.to_a-elms
         ##wikii~~ select=Others.new
         ##wikii~~ select.add else_ent
         ##wikii~~ elms << select if select.length>0
         return unless elms.size > 0

         elms.each {|sel|
            next unless sel.respond_to?("transformation=")
            pt = sel.transformation.origin
            tr = Geom::Transformation.new(pt)
            sel.transformation = tr
         }
         Sketchup.active_model.commit_operation
         return
      end
      angle = a.to_i if a
      bb = Sketchup.active_model.selection[0].bounds
      angle = -angle if dir == "cw"
      pt = bb.center
      Sketchup.active_model.start_operation("Rotate")

      elms = Sketchup.active_model.selection.find_all {|el| el.typename == "Group" or el.typename == "ComponentInstance"}
      else_ent=Sketchup.active_model.selection.to_a-elms
      select=Others.new
      select.add else_ent
      elms << select if select.length>0
      return unless elms.size > 0

      elms.each{|sel|
         next unless sel.respond_to?("transformation")
         pt = sel.transformation.origin
         #tr = Geom::Transformation.rotation(pt, [0, 0, 1], angle.degrees)
         tr = Geom::Transformation.rotation(pt, @axis, angle.degrees)
         if ctrl? #( @keys["17"] == true)
            if sel.is_a?(Others)
               ##wikii~~ old_trans=sel.transformation               
               ##wikii~~ definitions = $mm.definitions
               ##wikii~~ name=$mm.definitions.unique_name( "mover")
               ##wikii~~ tmp = definitions.add(name,sel.ents)
               
               ##wikii~~ moveto(sel,[0,0,0])
               ##wikii~~ p tmp
               ##wikii~~ tmp.entities.add_group(sel.ents[0])
               
               
               ##wikii~~ sel=$ee.add_instance(tmp,old_trans)
               ##wikii~~ $sel=sel
               ##wikii~~ tmp.entities.add_cpoint  old_trans.origin
               ##wikii~~ moveto(sel,old_trans.origin)
               sel=$ee.add_group(sel.ents)
               sel=sel.to_component
               $ss.add sel
               ##wikii~~ sel.transformation=old_trans 
            end
            c = sel.copy
         end
         sel.transform! tr
      }
      Sketchup.active_model.commit_operation
   end # rotate

   def drop(d,a)
      #puts "drop:#{a.inspect}"
      pt = nil
      Sketchup.active_model.start_operation("Drop")

      elms = Sketchup.active_model.selection.find_all {|el| el.typename == "Group" or el.typename == "ComponentInstance"}
      else_ent=Sketchup.active_model.selection.to_a-elms
      select=Others.new
      select.add else_ent
      elms << select if select.length>0
      return unless elms.size > 0

      elms.each { |e|
         next unless e.respond_to?("transformation")
         tr = e.transformation
         pt = tr.origin
         pt.z = 0
         if a == "true"
            ray = [e.transformation.origin, Z_AXIS.reverse]
            res =  Sketchup.active_model.raytest(ray)
            next unless res
            if res[0]
               #pt.z = res[0].z
               pt = res[0]
            end
         end
         tr = Geom::Transformation.new(pt)
         if e.respond_to?("transform!")
            #e.transform! tr
            #e.transformation = tr
            moveto(e, pt)
         end
      }
      Sketchup.active_model.commit_operation
   end # drop

end # class Mover


fn = File.basename(__FILE__, ".rb")
fn = "Mover3"

unless file_loaded?("#{fn}.rb")
   #UI.menu.add_item("Mover") { JF::Mover.create }
   UI.menu("plugins").add_item(fn.capitalize) {
   begin
   Mover3.new(fn)
   rescue => e
   s = "#{e}"
   body = e.backtrace.join("\n")
   UI.openURL("mailto:jim.foltz@gmail.com&subject=#{s}&body=#{body}")
   end
   }
   file_loaded "#{fn}.rb"
end
