# Name :        quick_selection.rb
# Description : Mimicks the Quick Selection of AutoCad
# Author :      D. Bur
# Usage :       1.Context Menu - Quick Selection
#		2.Fill the dialog box with filters
#               3. It loops until you hit the Cancel button
#               Number of selected objects is displayed in the status bar
# Date :        16.Jan.2oo6
# Type :        tool
# History:      beta 16.Jan.2oo6

require 'sketchup'
class Quick_select
   #defaults values
   @@how = ""
   @@through_what = "ģ"
   @@type = ""
   @@group = ""
   @@component = ""
   @@layer = ""
   @@material = ""
   @@bmaterial = ""

   def get_setting (*arg)
      @@type,@@group,@@component ,@@layer,@@material,@@bmaterial = *arg
   end

   def nothing_to_do
   end

   def build_visible_entities
      puts "Parsing searchable entities"
      ents = Sketchup.active_model.active_entities
      @@set_visible_entities = Set.new

      for e in ents
         case @@through_what
         when "ģ"
            @@set_visible_entities.insert e
         when "пɼ"
            if e.layer.visible? and not e.hidden?
               @@set_visible_entities.insert e
            end
         when "ɼͼ"
            if e.layer.visible?
               @@set_visible_entities.insert e
            end
         when "ɼ"
            if not e.hidden?
               @@set_visible_entities.insert e
            end
         when "ͼ"
            if not e.layer.visible?
               @@set_visible_entities.insert e
            end
         when ""
            if e.hidden?
               @@set_visible_entities.insert e
            end
         end
      end
   end

   def build_hidden_entities
      ents = Sketchup.active_model.active_entities
      @@set_hidden_entities = Set.new
      for e in ents
         if e.hidden?
            @@set_hidden_entities.insert e
         end
      end
   end

   def select_all_active
      puts "Selecting all"
      model = Sketchup.active_model
      for e in model.active_entities
         @@ss.add(e) if @@set_visible_entities.contains? e
      end
      Sketchup.set_status_text("All active objects selected: " + @@ss.length.to_s)
   end

   def select_me
      puts "Selecting with single filter: " + @@type
      model = Sketchup.active_model
      for e in model.active_entities
         if( yield(e) )
            case @@how
            when ""
               @@ss.add(e) if @@set_visible_entities.contains? e
            when ""
               @@ss.add(e) if @@set_visible_entities.contains? e
            when "Ƴ"
               @@ss.remove(e) if @@set_visible_entities.contains? e
            end
         end
      end
      Sketchup.set_status_text("Single filter: " + @@ss.length.to_s + " objects selected.")
   end

   def invert_selection
      puts "Inverting selection"
      ents = Sketchup.active_model.active_entities
      @@set_invert = Set.new
      for e in ents
         if not @@ss.contains? e
            @@set_invert.insert e if @@set_visible_entities.contains? e
         end
      end
      @@ss.clear
      @@ss.add(@@set_invert.to_a)
   end

   def put_in_set
      puts "Putting in temp set"
      ents = Sketchup.active_model.active_entities
      case @@set
      when "ty"
         s = @@set_type
      when "gn"
         s = @@set_group_name
      when "cn"
         s = @@set_component_name
      when "ln"
         s = @@set_layer
      when "mn"
         s = @@set_material
      when "bn"
         s = @@set_bmaterial
      end
      for e in ents
         if yield(e)
            s.insert e if @@set_visible_entities.contains? e
         end
      end
   end

   def put_in_set_material( side, m )
      puts "Putting in temp set " + side + " material: " + m
      ents = Sketchup.active_model.active_entities
      case @@set
      when "ty"
         s = @@set_type
      when "gn"
         s = @@set_group_name
      when "cn"
         s = @@set_component_name
      when "ln"
         s = @@set_layer
      when "mn"
         s = @@set_material
      when "bn"
         s = @@set_bmaterial
      end

      ents.each do |e|
         case side
         when "Front"
            if e.typename == "Face" and e.material and e.material.name == m
               s.insert(e) if @@set_visible_entities.contains? e
            end
         when "Back"
            if e.typename == "Face" and e.back_material and e.back_material.name == m
               s.insert(e) if @@set_visible_entities.contains? e
            end
         end
      end
   end

   def combine_sets
      puts "Combining sets"
      a_type = @@set_type.to_a
      a_group = @@set_group_name.to_a
      a_comp = @@set_component_name.to_a
      a_lay = @@set_layer.to_a
      a_mat = @@set_material.to_a
      a_bmat = @@set_bmaterial.to_a

      #~ puts "a_type=#{@@a_type.length}"
      puts "a_type=#{a_type.length}"
      puts "set_type=#{@@set_type.length}"
      puts "set_group_name=#{@@set_group_name.length}"
      puts "set_component_name=#{@@set_component_name.length}"
      result = []
      if not a_type.empty? and not a_group.empty?
         result = a_type & a_group
      else
         result = a_type if not a_type.empty?
         result = a_group if not a_group.empty?
         result = [] if a_type.empty? and a_group.empty?
      end
      if not result.empty? and not a_comp.empty?
         result = result & a_comp
      else
         result = a_comp if result.empty? and not a_comp.empty?
      end
      if not result.empty? and not a_lay.empty?
         result = result & a_lay
      else
         result = a_lay if result.empty? and not a_lay.empty?
      end
      if not result.empty? and not a_mat.empty?
         result = result & a_mat
      else
         result = a_mat if result.empty? and not a_mat.empty?
      end
      if not result.empty? and not a_bmat.empty?
         result = result & a_bmat
      else
         result = a_bmat if result.empty? and not a_bmat.empty?
      end
      return result
   end

   #------------------------------- Mixes @@ss and elligible set together to get final selection
   def update_selection( el )
      puts "Updating selection"
      el.each do |entity|
         if( @@ss.include?( entity ) )
            case @@how
            when ""
               (nothing_to_do)
            when ""
               @@ss.add(entity)
            when "Ƴ"
               @@ss.remove(entity)
            end
         else
            case @@how
            when ""
               @@ss.add(entity)
            when ""
               @@ss.add(entity)
            when "Ƴ"
               @@ss.remove(entity)
            end
         end
      end
   end

   def select_in_set
      puts "Parsing current set"
      for e in @@ss
         if( yield(e) )
            case @@how
            when ""
               @@ss.add(e) if @@set_visible_entities.contains? e
            when ""
               @@ss.add(e) if @@set_visible_entities.contains? e
            when "Ƴ"
               @@ss.remove(e) if @@set_visible_entities.contains? e
            end
         else
            case @@how
            when ""
               @@ss.remove(e) if @@set_visible_entities.contains? e
            when ""
               @@ss.remove(e) if @@set_visible_entities.contains? e
            end
         end
      end
   end

   def collect_groups_names
      g_list = []
      Sketchup.active_model.active_entities.each do |g|
         if g.typename == "Group" and g.name != ""
            g_list.push g.name
         end
      end
      return g_list
   end

   # No need to have unused components in the list
   def collect_components_names
      model = Sketchup.active_model
      components = model.definitions
      list = []
      components.each do |c|
         if c.count_instances != 0 and not c.name["Group"] and not c.name["Image"]
            list.push(c.name)
         end
      end
      list
   end

   def sel_curves
      puts "Selecting curves"
      ents = Sketchup.active_model.active_entities
      ents.each do |e|
         if e.kind_of?(Sketchup::Edge) and e.curve
            case @@how
            when ""
               @@ss.add(e.curve.edges)
            when ""
               @@ss.add(e.curve.edges)
            when "Ƴ"
               @@ss.remove(e.curve.edges)
            end
         end
      end
   end

   def sel_curves_in_set
      puts "Selecting curves in temp set"
      ents = Sketchup.active_model.active_entities
      ents.each do |e|
         if e.kind_of?(Sketchup::Edge) and e.curve
            case @@how
            when ""
               @@set_type.insert(e) if @@set_visible_entities.contains? e
            when ""
               @@set_type.insert(e) if @@set_visible_entities.contains? e
            when "Ƴ"
               @@set_type.delete(e) if @@set_visible_entities.contains? e
            end
         end
      end
   end


   def select_component_by_name n
      puts "Selecting components by name: " + n
      ents = Sketchup.active_model.active_entities
      #~ ents = Sketchup.active_model.active_entities
      ents.each do |e|
         if e.kind_of?(Sketchup::ComponentInstance) and e.definition.name == n
            case @@how
            when ""
               @@ss.add(e) if @@set_visible_entities.contains? e
            when ""
               @@ss.add(e) if @@set_visible_entities.contains? e
            when "Ƴ"
               @@ss.remove(e) if @@set_visible_entities.contains? e
            end
         end
      end
   end

   def select_group_by_name n
      puts "Selecting groups by name: " + n
      ents = Sketchup.active_model.active_entities
      ents.each do |e|
         if e.typename == "Group" and e.name == n
            case @@how
            when ""
               @@ss.add(e) if @@set_visible_entities.contains? e
            when ""
               @@ss.add(e) if @@set_visible_entities.contains? e
            when "Ƴ"
               @@ss.remove(e) if @@set_visible_entities.contains? e
            end
         end
      end
   end

   def select_material( side, m )
      puts "Selecting " + side + " material: " + m
      ents = Sketchup.active_model.active_entities
      ents.each do |e|
         case side
         when "Front"
            if e.typename == "Face" and e.material and e.material.name == m
               case @@how
               when ""
                  @@ss.add(e) if @@set_visible_entities.contains? e
               when ""
                  @@ss.add(e) if @@set_visible_entities.contains? e
               when "Ƴ"
                  @@ss.remove(e) if @@set_visible_entities.contains? e
               end
            end
         when "Back"
            if e.typename == "Face" and e.back_material and e.back_material.name == m
               case @@how
               when ""
                  @@ss.add(e) if @@set_visible_entities.contains? e
               when ""
                  @@ss.add(e) if @@set_visible_entities.contains? e
               when "Ƴ"
                  @@ss.remove(e) if @@set_visible_entities.contains? e
               end
            end
         end
      end
   end

   def select_material_default( side )
      puts "Selecting " + side + " default "
      ents = Sketchup.active_model.active_entities
      ents.each do |e|
         case side
         when "Front"
            if e.typename == "Face" and not e.material
               case @@how
               when ""
                  @@ss.add(e) if @@set_visible_entities.contains? e
               when ""
                  @@ss.add(e) if @@set_visible_entities.contains? e
               when "Ƴ"
                  @@ss.remove(e) if @@set_visible_entities.contains? e
               end
            end
         when "Back"
            if e.typename == "Face" and not e.back_material
               case @@how
               when ""
                  @@ss.add(e) if @@set_visible_entities.contains? e
               when ""
                  @@ss.add(e) if @@set_visible_entities.contains? e
               when "Ƴ"
                  @@ss.remove(e) if @@set_visible_entities.contains? e
               end
            end
         end
      end
   end

   def select
      model = Sketchup.active_model
      model.start_operation "Quick selection"
      @@stop_it = false
      entities = model.active_entities
      # Current selection set
      @@ss = model.selection

      # Components, Layers, Materials names
      layers = model.layers
      materials = model.materials
      components = model.definitions
      layers_names = layers.collect {|l| l.name}
      materials_names = materials.collect {|m| m.name}
      materials_names.unshift("<Default>")
      materials_names.unshift("")
      materials_display_names = materials.collect {|m| m.display_name}
      materials_display_names.unshift("Default")

      groups_names = (collect_groups_names)
      components_names = (collect_components_names)



      # Dialog box
      #~ what_list = ["Start new", "Add to current", "Remove from current", "Invert", "Memorize", "Recall"].join("|")
      #~ what_list =%w{  Ƴ ѡ  ָ}.join("|")
      what_list =%w{  Ƴ ѡ}.join("|")
      #~ through_list = ["Whole model", "All visible objects", "Visible layers", "Visible objects", "Hidden layers", "Hidden objects"].join("|")
      through_list = ["ģ", "пɼ", "ɼͼ", "ɼ", "ͼ", ""].join("|")
      #~ types_list = ["", "Construction point", "Construction line", "Edge", "Curve", "Face", "Group", "Component", "Dimension", "Text", "Image", "Section Plane"].join("|")
      types_list = ["", "", "", "", "", "", "Ⱥ", "", "ߴ", "ı", "ͼ", ""].join("|")
      groups_list = groups_names.unshift("").join("|")
      components_list = components_names.unshift("").join("|")
      layers_list = layers_names.unshift("").join("|")
      materials_list = materials_display_names.unshift("").join("|")

      dropdowns = [what_list, through_list, types_list, groups_list, components_list, layers_list, materials_list,materials_list]

      #~ prompts = ["Selection setSelection set:  ", "Search through:  ", "Objects:  ", "Group name:  ", "Component name:  ", "On layer:  ", "Front material:  ", "Back material:  "]
      prompts = ["ѡʽ:  ", "ѡΧ:  ", "ʵ:  ", "Ⱥ:  ", ":  ", "ͼ:  ", ":  ", ":  "]
      values = [@@how,@@through_what,@@type,@@group,@@component,@@layer,@@material, @@bmaterial]
      results = inputbox prompts, values, dropdowns, "ѡ"
      @@stop_it = true if not results
      return if not results
      @@how = results[0].convert
      @@through_what = results[1].convert
      @@type = results[2].convert
      @@group = results[3].convert
      @@component = results[4].convert
      @@layer = results[5].convert
      @@material = results[6].convert
      @@bmaterial = results[7].convert

      # Entities init
      #Visible entities
      (build_visible_entities)

      # Selection init
      case @@how
      when ""
         @@ss.clear
      when "Ƴ"
         entities = @@ss
      end

      # Invert, Memorize or Recall
      case @@how
      when "ѡ"
         (invert_selection)
         #~ when ""
         #~ $selectionSet.memorize
         #~ when "ָ"
         #~ $selectionSet.recall
      end

      # Check filters
      nb_filters = (results.uniq.length - 3)

      # No, single or multiple filter
      case nb_filters
      when 0
         @@ss = (select_all_active)
      when 1
         # Object type filter
         if ( @@type!= "" )
            case @@type
            when ""
               select_me {|e| e.kind_of?(Sketchup::ConstructionPoint)}
            when ""
               select_me {|e| e.kind_of?(Sketchup::ConstructionLine)}
            when ""
               select_me {|e| e.kind_of?(Sketchup::Edge)}
            when ""
               sel_curves
            when ""
               select_me {|e| e.kind_of?(Sketchup::Face)}
            when "Ⱥ"
               select_me {|e| e.typename == "Group"}
            when ""
               select_me {|e| e.kind_of?(Sketchup::ComponentInstance)}
            when "ߴ"
               select_me {|e| (e.typename == "DimensionLinear")}
               select_me {|e| (e.typename == "DimensionRadial")}
            when "ı"
               select_me {|e| e.kind_of?(Sketchup::Text)}
            when "ͼ"
               select_me {|e| e.kind_of?(Sketchup::Image)}
            when ""
               select_me {|e| e.kind_of?(Sketchup::SectionPlane)}
            end
         end

         # Group name filter
         if ( @@group != "" )
            select_group_by_name @@group
         end

         # Component name filter
         if ( @@component != "" )
            select_component_by_name @@component
         end
         # Layer name filter
         if ( @@layer != "" )
            select_me {|e| e.layer.name == @@layer}
         end
         # Material filter
         case @@material
         when ""
            (nothing_to_do)
         when "Default"
            (select_material_default "Front")
         else
            index = materials_display_names.index(@@material)
            materialname = index ? materials_names[index] : "Default"
            (select_material "Front", materialname)
         end

         # Back material filter
         case @@bmaterial
         when ""
            (nothing_to_do)
         when "Default"
            (select_material_default "Back")
         else
            index = materials_display_names.index(@@bmaterial)
            bmaterialname = index ? materials_names[index] : "Default"
            (select_material "Back", bmaterialname)
         end
         Sketchup.set_status_text("Single filter: " + @@ss.length.to_s + " objects selected.")
      else

         # Combine filters
         # Object type filter
         # Init sets
         @@set_type = Set.new
         @@set_group_name = Set.new
         @@set_component_name = Set.new
         @@set_layer = Set.new
         @@set_material = Set.new
         @@set_bmaterial = Set.new
         combined_set = []

         if ( @@type!= "" )
            @@set = "ty"
            case @@type
            when ""
               put_in_set {|e| e.kind_of?(Sketchup::ConstructionPoint)}
            when ""
               put_in_set {|e| e.kind_of?(Sketchup::ConstructionLine)}
            when ""
               put_in_set {|e| e.kind_of?(Sketchup::Edge)}
            when ""
               sel_curves_in_set
            when ""
               put_in_set {|e| e.kind_of?(Sketchup::Face)}
            when "Ⱥ"
               put_in_set {|e| e.typename == "Group"}
            when ""
               put_in_set {|e| e.kind_of?(Sketchup::ComponentInstance)}
            when "ߴ"
               put_in_set {|e| (e.typename == "DimensionLinear")}
               put_in_set {|e| (e.typename == "DimensionRadial")}
               put_in_set {|e| e.kind_of?(Sketchup::Text)}
            when "ͼ"
               put_in_set {|e| e.kind_of?(Sketchup::Image)}
            when ""
               put_in_set {|e| e.kind_of?(Sketchup::SectionPlane)}
            end
         end

         # Group name filter
         if ( @@group != "" )
            @@set = "gn"
            select_group_by_name @@group
         end

         # Component name filter
         if ( @@component != "" )
            @@set = "cn"
            select_component_by_name @@component
         end
         # Layer name filter
         if ( @@layer != "" )
            @@set = "ln"
            put_in_set {|e| e.layer.name == @@layer }
         end

         # Material filter
         case @@material
         when ""
            (nothing_to_do)
         when "Default"
            (select_material_default "Front")
         else
            @@set = "mn"
            index = materials_display_names.index(@@material)
            materialname = index ? materials_names[index] : "Default"
            #(select_material "Front", materialname)
            (put_in_set_material "Front", materialname)
         end
         # Back material filter
         case @@bmaterial
         when ""
            (nothing_to_do)
         when "Default"
            (select_material_default "Back")
         else
            @@set = "bn"
            index = materials_display_names.index(@@bmaterial)
            bmaterialname = index ? materials_names[index] : "Default"
            #(select_material "Back", bmaterialname)
            (put_in_set_material "Back", bmaterialname)
         end


         combined_set = (combine_sets)
         puts combined_set.length
         (update_selection combined_set)

         Sketchup.set_status_text("Multiple filter: " + @@ss.length.to_s + " objects selected.")
      end
      model.commit_operation
   end

end #class

def qss
   puts "ѡʼ..."
   model = Sketchup.active_model
   #model.start_operation "Quick selection"
   #@@stop_it = true
   #while not @@stop_it
   Quick_select.new.select
   #  end
   Sketchup.set_status_text(" " + Sketchup.active_model.selection.length.to_s + " ʵ屻ѡ")
   #model.commit_operation
   puts "ѡ..."
end

#~ $selectionSet=Selection.new

# Add some menu items
#if not file_loaded?("quick_selections.rb")
####### Add a context menu handler that will add a menu choice to a context menu
#  UI.add_context_menu_handler do |menu|
#    menu.add_item("Quick selection...") { qss }
#    submenu=menu.add_submenu("Selection Memory")
#		submenu.add_item("Memorize") { $selectionSet.memorize }
#		submenu.add_item("Add") { $selectionSet.add }
#		submenu.add_item("Recall") { $selectionSet.recall }
#  end
#
#  UI.menu("Plugins").add_separator
#  UI.menu("Plugins").add_item("Quick selection...") { qss }
#end
##wikii~~ file_loaded("quick_selections.rb")


def qqsb
   types={"ConstructionPoint"=>"",
      "ConstructionLine"=>"",
      "Edge"=>"",
      "Curve"=>"",
      "Face"=>"",
      "Group"=>"Ⱥ",
      "ComponentInstance"=>"",
      "DimensionLinear"=>"ߴ",
      "DimensionRadial"=>"ߴ",
      "Text"=>"ı",
      "Image"=>"ͼ",
      "SectionPlane"=>"",
   }
   if e=Sketchup.active_model.selection[0]
      type=types[e.typename]
      group= e.typename== "Group" ? e.name : ""
      component= e.typename=="ComponentInstance" ? e.definition.name : ""
      layer=e.layer.name
      material= (e.typename=="Face" and e.material) ? e.material.name[1..-2] : ""
      bmaterial= (e.typename=="Face"and e.back_material) ? e.back_material.name[1..-2] : ""
      #~ qss
      puts "ѡʼ..."
      model = Sketchup.active_model
      #model.start_operation "Quick selection"
      #@@stop_it = true
      #while not @@stop_it
      tmp=Quick_select.new
      tmp.get_setting type,group,component ,layer,material,bmaterial
      tmp.select
      #  end
      Sketchup.set_status_text(" " + Sketchup.active_model.selection.length.to_s + " ʵ屻ѡ")
      #model.commit_operation
      puts "ѡ..."
   else
      qss
   end
end

def quick_select
   if ctrl?
      qqsb
   else
      qss
   end
end
