#        Name : clean.rb
# Description :
#      Author : Didier Bur
#       Usage :
#        Date : 11.2005
#        Type : Model aufrumen und Filtertool

require 'sketchup.rb'


#-----------------------------------------------------------------------------------------------------
#                                        Modell aufrumen
#-----------------------------------------------------------------------------------------------------

# --------------------------------------------------- entfernt alle Kostruktionspunkte
def deleteConPointsOnly

   model = Sketchup.active_model
   entities = model.active_entities

   model.start_operation "deleteConPointsOnly"

   8.times do |something| ### runs 8 times as entities x 1 can miss out some bits...
      entities.each do |entity|
         if entity.typename == "ConstructionPoint"
            entity.erase!
         end
      end
   end

   model.commit_operation
end
#---------------------------------------------------entfernt alle Konstruktionslinien
def exLines
   model=Sketchup.active_model
   sel=model.selection
   ents=model.entities
   defs=model.definitions
   x=[]
   model.start_operation "exLines"
   count=0
   for e in ents
      if e.class==Sketchup::ConstructionLine
         x.push(e)
         count+=1
      end
   end
   for d in defs
      ents=d.entities
      for de in ents
         if de.class==Sketchup::ConstructionLine
            x.push(de)
            count+=1
         end
      end
   end
   x.each {|xx| xx.erase!}
   model.commit_operation
   puts "Erased "+count.to_s+" construction lines."
end
#------------------------------------------------------zeigt die offenen Linienenden durchnummeriert an
class BadDrawing

   def BadDrawing.labelLines
      allverts  = Array.new
      lonepoints = Array.new
      am = Sketchup.active_model        # Sketchup Active Model.
      se = am.entities                  # Sketchup Entities.
      ae = am.selection                 # Work with selection, if any...
      if ae.length == 0 then ae = se end     # ...else work with whole model.
      ae.each {|e|
         if  e.class == Sketchup::Edge then   # If this is an edge...
            e.vertices.each {|v|               # ...loop through its vertices... and
               if v.edges.length == 1 then lonepoints << v.position end   # ...see if there is only 1 connected edge.
            }
         end   # if
      }
      print "There are ", lonepoints.length, " open line segments.\n"
      am.start_operation "Label Stray Lines"        # Encapsulate into a single UNDO operation.
      i=1
      lonepoints.each {|p|
         se.add_text(i.to_s << " of " << lonepoints.length.to_s , p, [0,0,5] )    # Add the Text Label.
         i+=1
      }
      am.commit_operation                           # End of UNDO capsule.
   end  # def
end    # class
#------------------------------------------------------subert das Modell von nicht geometrisch notwendigen Linien
class Cleanup

   def self.cleanup_model
      cleanup_entities(Sketchup.active_model.active_entities)
   end
   def self.cleanup_selected
      cleanup_entities(Sketchup.active_model.selection)
   end
   def self.cleanup_entities(entities)
      model = Sketchup.active_model
      model.start_operation "Cleanup Operation"

      # Erase everything that can be erased.  Is this too simple? :-)
      model.active_entities.erase_entities entities.find_all { |e| can_be_erased(e) }

      # Now we are done and we can end the operation
      model.commit_operation
   end

   private
   def self.can_be_erased(entity)
      # We only consider edges
      return false unless entity.kind_of?(Sketchup::Edge)

      faces = entity.faces

      # Delete the edge if it doesn't contribute to a face
      return true if faces.length == 0;
      # If it contributes to more than one face, it can not be removed
      return false if faces.length != 2;

      # It must contribute to only 2 faces to be a canidate.
      # If someone can think of a situation where an edge could
      # contribute to more than 2 faces and still be a canidate
      # for removal, please let me know.

      normal0 = faces[0].normal
      normal1 = faces[1].normal

      dot = normal0.dot normal1

      # What should the tolerance be on the dot product
      # for two faces to be co-planar?

      # Check for two faces pointing in the same direction
      if dot > 0.99999
         # Check that the materials are the same
         return true if same_side_material?(faces[0],faces[1])
      end
      # Check for co-planar faces, but reversed normals
      if dot < -0.99999
         # Check that the material are the same
         return true if opposite_side_material?(faces[0],faces[1])
      end
   end

   def self.same_side_material?(face1,face2)
      face1.material == face2.material and face1.back_material == face2.back_material
   end
   def self.opposite_side_material?(face1,face2)
      face1.material == face2.back_material and face1.back_material == face2.material
   end

end  #end class
#---------------------------------------------------------repariert Linien
def repair_broken_lines
   model = Sketchup.active_model
   #define everything as one undo operation
   model.start_operation "Repair Broken Lines"

   trunk = Array.new
   ss = model.selection
   #search all edges in selection set
   for e in ss
      if (e.kind_of?(Sketchup::Edge))
         p=e.parent
         #take both vertices
         for v in e.vertices
            #take all edges connected to v (including e)
            es=v.edges
            #when two edges are connected
            if (es.length==2)
               vec1 = es[0].line[1]
               vec2 = es[1].line[1]
               #and both are in the same direction
               if (vec1.parallel?(vec2))
                  #then the two lines can be repaired
                  #this is done by generating a new line from the vertex to a random point
                  l = p.entities.add_line (v.position, Geom::Point3d.new(3.45,6.78,9.12))
                  #and put that line in a trunk
                  trunk.push(l)
               end
            end
         end
      end
   end

   #erase all lines in trunk
   total = trunk.length
   for l in trunk
      l.erase!
   end
   model.commit_operation
   UI.messagebox ("#{total} 条线被修复")
end

#---------------------------------------------------------verschweit Linien

def join_edges
   model=Sketchup.active_model
   ents=model.active_entities
   sel=model.selection
   sl=sel.length
   verts=[]
   edges=[]
   newVerts=[]
   startEdge=startVert=nil

   #CHECK FOR A VALID SELECTION

   #	if sel.first.curve
   #		return UI.messagebox("Please select at least 2 non-joined edges") if sel.length == sel.first.curve.count_edges
   #	end

   #DELETE NON-EDGES, GET THE VERTICES

   sel.each {|item| edges.push(item) if item.typename=="Edge"}
   edges.each {|edge| verts.push(edge.vertices)}
   verts.flatten!
   return UI.messagebox("请至少选择两条非重合的线") if edges.length < 2

   #FIND AN END VERTEX

   vertsShort=[]
   vertsLong=[]
   verts.each do |v|
      if vertsLong.include?(v)
         vertsShort.push(v)
      else
         vertsLong.push(v)
      end
   end
   if (startVert=(vertsLong-vertsShort).first)==nil
      startVert=vertsLong.first
      closed=true
      startEdge = startVert.edges.first
   else
      closed=false
      startEdge = (edges & startVert.edges).first
   end
   #sel.clear


   #SORT VERTICES, LIMITING TO THOSE IN THE SELECTION SET

   if startVert==startEdge.start
      newVerts=[startVert]
      counter=0
      while newVerts.length < verts.length
         edges.each do |edge|
            if edge.end==newVerts.last
               newVerts.push(edge.start)
            elsif edge.start==newVerts.last
               newVerts.push(edge.end)
            end
         end
         counter+=1
         if counter > verts.length
            return nil if UI.messagebox("似乎出现了一点小错误,再试一次?", MB_YESNO)!=6
            newVerts.reverse!
            reversed=true
         end
      end
   else
      newVerts=[startVert]
      counter=0
      while newVerts.length < verts.length
         edges.each do |edge|
            if edge.end==newVerts.last
               newVerts.push(edge.start)
            elsif edge.start==newVerts.last
               newVerts.push(edge.end)
            end
         end
         counter+=1
         if counter > verts.length
            return nil if UI.messagebox("似乎出现了一点小错误,再试一次", MB_YESNO)!=6
            newVerts.reverse!
            reversed=true
         end
      end
   end
   newVerts.uniq!
   newVerts.reverse! if reversed


   #CONVERT VERTICES TO POINT3Ds

   newVerts.collect!{|x| x.position}
   if closed
      newVerts.push(newVerts[0])
   else
      newVerts.push(newVerts[0]) if UI.messagebox("是否闭合曲线呢?", MB_YESNO)==6
      closed=true
   end

   #CREATE THE CURVE

   model.start_operation "weld"
   edges.each {|item| item.explode_curve if item.curve and newVerts.include?(item.start.position)}
   #sel.each {|e| e.erase!}
   gp1e=(gp1=ents.add_group).entities
   curveEdges=gp1e.add_curve(newVerts)
   $gpents=gp1.explode
   ents[-1].find_faces if closed and ents[-1].typename=="Edge" and UI.messagebox("请问您是否要生成面呢?", MB_YESNO, "Weld Edges Utility")==6
   #	if $gpents.first.typename=="Edge"
   #		edges=$gpents.first.curve.edges
   #	elsif $gpents.first.typename=="Curve"
   #		edges=$gpents.first.edges
   #	else
   #		edges=$gpents.last.edges
   #	end
   #	model.active_entities.add_curve(newVerts)
   #	edges.each_index do |i|
   #		if i>0
   #			edges[i].reverse! if edges[i].start!=edges[i-1].end
   ##			if edges[i].start!=edges[i-1].end
   ##				sel.clear
   ##				sel.add(edges)
   ##				join_edges
   ##				return
   ##			end
   #		end
   #	end
   model.commit_operation
   sel.clear
end

#--------------------------------------------------------- verlngert eine Linie zum Anstosspunkt
class Extend

   def Extend.nearest 	#EXTEND TO NEAREST EDGE
      model=Sketchup.active_model
      view=model.active_view
      ents=model.entities
      ss=model.selection
      sel=[]
      0.upto(ss.length-1) do |a|
         sel.push(ss[a]) if ((ss[a].class == Sketchup::Edge) && (ss[a].curve == nil))
      end
      model.start_operation "Extend"
      0.upto(sel.length-1) do |a|
         d1=d2=nil
         s=sel[a]
         pt1=s.start
         pt2=s.end
         vec1=s.line[1].reverse
         vec2=s.line[1]
         ext1=model.raytest(pt1.position,vec1)
         ext2=model.raytest(pt2.position,vec2)
         d1=(pt1.position.distance (ext1[0])).to_f if ext1 != nil
         d2=(pt2.position.distance (ext2[0])).to_f if ext2 != nil
         if ((d1 != nil) && (d2 != nil))
            if d1 <= d2
               t=Geom::Transformation.new(ext1[0]-pt1.position)
               ents.transform_entities(t, pt1)
            else
               t=Geom::Transformation.new(ext2[0]-pt2.position)
               ents.transform_entities(t, pt2)
            end
         elsif ((d1 != nil) || (d2 != nil))
            if d1 != nil
               t=Geom::Transformation.new(ext1[0]-pt1.position)
               ents.transform_entities(t, pt1)
            else
               t=Geom::Transformation.new(ext2[0]-pt2.position)
               ents.transform_entities(t, pt2)
            end
         end
      end
      ss.clear()
      model.commit_operation
   end

   def Extend.validate_selection
      Sketchup.active_model.selection.each {|e| return true if e.class == Sketchup::Edge}
      return nil
   end

end	#END CLASS

#-------------------------------------------------------------Flchen umkehren mit Mausklick

class ReverseFaceTool

   def initialize
      $ip = Sketchup::InputPoint.new                   # Define the variable.
   end


   def reset
      $ip.clear
      Sketchup::set_status_text "Left Click any Face to Reverse"   # Status Bar hint.
   end

   def activate
      self.reset
   end

   def onMouseMove(flags, x, y, view)
      $ip.pick(view,x,y)                    # Get current Screen coordinates.
      view.tooltip = $ip.tooltip            # Set a tooltip if hovering over geometry.
   end


   def onLButtonDown(flags, x, y, view)
      $ip.pick(view,x,y)                       # Get current screen coordinates.
      face = $ip.face                          # If hovering over a face, this will return the face definition.
      if (face != nil) then face.reverse! end  # if not nil (we have a face), then reverse the face.
   end

end # class ReverseFaceTool

#-------------------------------------------------------------subert die Fenster von nicht gebrauchtem
class PurgeAll ### v1.3
   def PurgeAll::do
      ### show VCB and status info
      Sketchup::set_status_text(("Purge..." ),SB_PROMPT)
      Sketchup::set_status_text("",SB_VCB_LABEL)
      Sketchup::set_status_text("",SB_VCB_VALUE)
      ###
      if Sketchup.version_number < 6000277###v1.1
         Sketchup::set_status_text(("Purge... ERROR..." ),SB_PROMPT)
         UI.beep
         UI.messagebox("Sorry, but 'Purge...'\nis for Version 6 + only. ")
         return nil
      end#if
      ### dialog ###v1.2
      @compos="Yes" if not @compos
      @layers="Yes" if not @layers
      @maters="Yes" if not @maters
      @styles="Yes" if not @styles
      @report="Yes" if not @report
      prompts=[": ","ͼ: ",": ",": ","ɱ: "]
      values=[@compos,@layers,@maters,@styles,@report]
      yn="Yes|No"###v1.4
      popups=[yn,yn,yn,yn,yn]
      title=""
      results=inputbox(prompts,values,popups,title)
      return nil if not results
      ### do processing of results
      @compos=results[0]
      @layers=results[1]
      @maters=results[2]
      @styles=results[3]
      @report=results[4]
      ### ###v1.2
      model=Sketchup.active_model
      entities=model.entities
      definitions=model.definitions
      layers=model.layers
      materials=model.materials
      styles=model.styles
      model.start_operation "Purge..."
      ###
      if @compos=="Yes" ###v1.2
         compos1=definitions.count
         definitions.purge_unused
         co=compos1-definitions.count
         cmsg="\n:\t "+co.to_s
      else
         cmsg="\n..."
      end#if
      if @layers=="Yes" ###v1.2
         layers1=layers.count
         layers.purge_unused
         la=layers1-layers.count
         lmsg="\nͼ: "+la.to_s
      else
         lmsg="\n..."
      end#if
      if @maters=="Yes" ###v1.2
         materials1=materials.count
         materials.purge_unused
         ma=materials1-materials.count
         mmsg="\n:\t "+ma.to_s
      else
         mmsg="\n..."
      end#if
      if @styles=="Yes" ###v1.2
         styles1=styles.count
         styles.purge_unused
         st=styles1-styles.count
         smsg="\n:\t "+st.to_s+"\n"
      else
         smsg="\n...\n"
      end#if
      ### report v1.3
      if @compos=="No" && @layers=="No"
         if @maters=="No" && @styles=="No"
            UI.messagebox("ɶҸﰡ\t")
            return nil
         end#if
      end#if
      if @report=="Yes"
         UI.messagebox("ϲ!ͨҵŬǹ:\n"+cmsg+lmsg+mmsg+smsg+"\t\t\t")
      end#if
      ###
      model.commit_operation###for undo
   end #def
end #class
#-------------------------------------------------------------Material entfernen
class RemMats

   def parse_ent(ent)
      if ent.is_a? Sketchup::ComponentInstance then
         ent.definition.entities.each {|ede| parse_ent(ede) }
      elsif (ent.is_a? Sketchup::Group) then ent.entities.each {|ege| parse_ent(ege) }
      end ; # if
      ent.material      = nil if ent.respond_to?(:material) ;
      ent.back_material = nil if ent.respond_to?(:back_material) ;
   end  # def parse_ent

   def remove() ;
      Sketchup.active_model.selection.each {|e| parse_ent(e) ; }
   end ;  # def remove()
end ;    # class RemMats

def remove_Materials_Smustard

   if Sketchup.active_model.selection.empty? then
      UI.messagebox("˵ǲص,ɶѡͽɾ,Ҫȫɾֲ" ) ;
      return ;
   end ;
   Sketchup.active_model.start_operation("Delete All Materials") ;
   x = RemMats.new ;
   x.remove ;
   Sketchup.active_model.commit_operation ;

end ;
#-------------------------------------------------------------setzt Text, Bemaung etc auf Layer

def dispatch_objects
   model=Sketchup.active_model
   ents=model.active_entities

   model.start_operation "Dispatch objects"

   inputbox1 = %w[Yes No].join("|")
   inputbox2 = %w[Yes No].join("|")
   inputbox3 = %w[Yes No].join("|")
   inputbox4 = %w[Yes No].join("|")
   dropdowns = [inputbox1, inputbox2, inputbox3, inputbox4]

   # sets the default settings
   $texts = "Yes" if not $texts
   $dims = "Yes" if not $dims
   $sect = "Yes" if not $sect
   $cons = "Yes" if not $cons

   prompts = ["文字注释","尺寸标注","剖切面","构造线及构造点  "]
   values = [$texts, $dims, $sect, $cons]
   $texts, $dims, $sect, $cons = inputbox prompts,values, dropdowns, "分类设置"
   #results = inputbox prompts, values, "Select objects types"
   #return if not results

   countText=countTextMoved=0
   countDim=countDimMoved=0
   countSect=countSectMoved=0
   countCons=countConsMoved=0

   if( $texts == "Yes" )
      txtLayer=model.layers.add("文字注释").name
      for e in ents
         if e.class==Sketchup::Text
            countText+=1
            #puts e.layer.name
            if e.layer.name != txtLayer
               e.layer=txtLayer
               countTextMoved+=1
            end
         end
      end
   end

   if( $dims == "Yes" )
      dimLayer=model.layers.add("尺寸标注").name
      for e in ents
         #if( e.class==Sketchup::DimensionLinear ) or ( e.class==Sketchup::DimensionRadial )
         if( e.typename == "DimensionLinear" ) or ( e.typename == "DimensionRadial" )
            countDim+=1
            #puts e.layer.name
            if e.layer.name != dimLayer
               e.layer=dimLayer
               countDimMoved+=1
            end
         end
      end
   end

   if( $sect == "Yes" )
      sectLayer=model.layers.add("剖切面").name
      for e in ents
         if e.typename == "SectionPlane"
            countSect+=1
            #puts e.layer.name
            if e.layer.name != sectLayer
               e.layer=sectLayer
               countSectMoved+=1
            end
         end
      end
   end

   if( $cons == "Yes" )
      consLayer=model.layers.add("构造线及构造点").name
      for e in ents
         if( e.typename == "ConstructionLine" ) or ( e.typename == "ConstructionPoint" )
            countCons+=1
            #puts e.layer.name
            if e.layer.name != consLayer
               e.layer=consLayer
               countConsMoved+=1
            end
         end
      end
   end

   UI.messagebox( countText.to_s + " texts items found, " + countTextMoved.to_s + " moved to layer \"Text\".\n" + countDim.to_s + " dims items found, " + countDimMoved.to_s + " moved to layer \"Dims\".\n" + countSect.to_s + " sections planes items found, " + countSectMoved.to_s + " moved to layer \"Section Planes\".\n" + countCons.to_s + " construction items found, " + countConsMoved.to_s + " moved to layer \"Construction Geometry\"." )

   model.commit_operation

end

if( not file_loaded?("dispatch_objects.rb") )
   add_separator_to_menu("Plugins")
   UI.menu("Plugins").add_item("Dispatch Objects") { dispatch_objects }
end

#-------------------------------------------------------------archiviert das aktuelle Projekt
class Archiver

   TOO_BIG = 1024 * 1024 * 2 # 2MB ???

   ###---------------------------------------

   def Archiver::make

      model = Sketchup.active_model
      entities = model.entities
      mpath = model.path
      paths = model.path.split("\\")
      paths = paths - [paths.last]
      path = paths.join("\\")+"\\"
      title = model.title

      ###[7, 41, 21, 18, 12, 2005, 0, 352, false, "GMT Standard Time"]
      t = Time::now.to_a
      sfix = t[5].to_s[2..4]+"-"+t[4].to_s+"-"+t[3].to_s+"-"+t[2].to_s+"-"+t[1].to_s+"-"+t[0].to_s

      name = title+"["+sfix+"].skp"
      archive = path+"\\Archives"

      Dir.mkdir(archive) if not FileTest.directory?(archive)

      archname = archive+"\\"+name

      model.save name+".tmp"

      Archiver.copy((path+name+".tmp"),archname)

      File.delete(path+name+".tmp")

      UI.messagebox("   Archiv:\n\n#{name}\n\n    in:\n\n#{archive}\\   \n\n")

      #-----------------------
   end#def

   # copy files
   def Archiver::catname (from, to)
      if FileTest.directory? to
         to +
         if to =~ /\\/
            if to[-1,1] != '\\' then '\\' end + basename(from)
            else
               if to[-1,1] != '/' then '/' end + basename(from)
               end
            else
               to
            end
         end
         ###
         def Archiver::syscopy (from, to)

            to = Archiver.catname(from, to)

            fsize = File.size(from)
            fsize = 1024 if fsize < 512
            fsize = TOO_BIG if fsize > TOO_BIG

            fmode = File.stat(from).mode
            tpath = to

            from = File.open(from, "r")
            from.binmode
            to = File.open(to, "w")
            to.binmode

            begin
               while true
                  r = from.sysread(fsize)
                  rsize = r.size
                  w = 0
                  while w < rsize
                     t = to.syswrite(r[w, rsize - w])
                     w += t
                  end
               end
            rescue EOFError
               ret = true
            rescue
               ret = false
            ensure
               to.close
               from.close
            end
            File.chmod(fmode, tpath)
            ret
         end
         ###
         def Archiver::copy (from, to, verbose = false)
            $stderr.print from, " --> ", catname(from, to), "\n" if verbose
            Archiver.syscopy from, to
         end
         ###


         #-----------------------
      end#class
