# Supports Organizer.rb
=begin rdoc
= Centerpoint Finder
Copyright 2009, Chris Fullmer
All Rights Reserved

== Disclaimer
THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

== License
This software is distributed under the Smustard End User License Agreement
http://www.smustard.com/eula

== Information
Author:: Chris Fullmer
Organization:: www.ChrisFullmer.com and distributed on www.sketchucation.com
Name:: Centerpoint Finder
Version:: v1.0
SU Version:: Developed on SU 7, should be compatible with 6 and 5 at least.
Date:: May 11, 2009
Description:: This scipt places a construction point at the centerpoint of an arc, even after the arc has been exploded in SketchUp.
Usage:: Select two line segments that used to be part of an arc object in SketchUp.  Then run the plugin from Plugins>Chris Fullmer Tools>
History::
1.0:: 2009-05-11
   * First release
=end


require 'sketchup.rb'

module Clf_centerpoint_finder

def Clf_centerpoint_finder.main
	model = Sketchup.active_model
	sel = model.selection
	ents = model.entities
	if sel.length != 2
		UI.messagebox "Please select two lines segments before running this script"
		return
	end

	if sel[0].typename == "Edge" && sel[0].typename == "Edge"
		seg1 = sel[0]
		seg2 = sel[1]
		s1p1 = seg1.start.position
		s1p2 = seg1.end.position
		s2p1 = seg2.start.position
		s2p2 = seg2.end.position
		orig_vec1 = seg1.line[1]
		orig_vec2 = seg2.line[1]
		mp1 = Geom.linear_combination 0.5, s1p1, 0.5, s1p2
		mp2 = Geom.linear_combination 0.5, s2p1, 0.5, s2p2
		if orig_vec1.parallel? orig_vec2
			UI.messagebox "Segments are parallel"
			return
		end
		zvec = orig_vec1.cross orig_vec2
		vec1 = orig_vec1.cross zvec
		vec2 = orig_vec2.cross zvec
		
		line1 = [mp1, vec1]
		line2 = [mp2, vec2]
		pt = Geom.intersect_line_line(line1, line2)
		if pt == nil
			puts "pt1 = nil"
			vec1.reverse!
			vec2.reverse!
			line1 = [mp1, vec1]
			line2 = [mp2, vec2]
			pt = Geom.intersect_line_line(line1, line2)
		end
		if pt == nil
			puts "pt2 is nil also."
			UI.messagebox "These segments do not have a centerpoint"
			return
		end
		model.start_operation "Centerpoint Finder"
		ents.add_cpoint pt
		model.commit_operation
	else
		UI.messagebox "Please select two lines segments before running this script"
		return
	end
end # main
end # module

if( $submenu )
	organizer = $submenu 
	if !file_loaded?(__FILE__) then
		organizer.add_item("Arc Centerpoint Finder") { Clf_centerpoint_finder.main  }
	end
else
	if !file_loaded?('clf_menu_loader') then
		$clf_tools_menu = UI.menu("Plugins").add_submenu("Chris Fullmer Tools")
	end
	if !file_loaded?(__FILE__) then
		$clf_tools_menu.add_item("Arc Centerpoint Finder") { Clf_centerpoint_finder.main  }
	end
end

file_loaded('clf_menu_loader')
file_loaded(__FILE__)