# Supports Organizer.rb
=begin rdoc
= Scale and Rotate Multiple
Copyright 2009, Chris Fullmer
All Rights Reserved

== Disclaimer
THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

== License
This software is distributed under the Smustard End User License Agreement
http://www.smustard.com/eula

== Information
Author:: Chris Fullmer
Organization:: www.ChrisFullmer.com and distributed on www.sketchucation.com
Name:: Scale and Rotate Multiple
Version:: v2.5
SU Version:: Developed on SU 7, should be compatible with 6 and 5 at least.
Date:: December 22, 2009
Description:: This script takes a selection of groups/components and scales them based on their 3d centerpoint, the world axis, their axis location, or their "base" (which is the center of their base for those of us who don't set our axes).
Usage:: Select the components and/or groups to be scaled.  Run the plugin at Plugins>Chris Fullmer Tools>Scale Multiple> .  There are two plugins found here.  One will scale everything equally.  The other will scale them randomly based on minimum and maximum provided values.  NOTE: The scale factors are not percentages.  They are factors.  Which means if you want to scale something to be twice as large, you enter 2.  If you want something to be one half the size, enter .5 .  Entering 50 will not a result that is 50% the original, but will give a result that is 50 times larger than the original.
Requires::  No additional scripts
History::
1.0:: 2009-03-13
   * First release
2.0:: 2009-03-22
   * Fixed a bug where it didn't let you scale items more than once using Axis.  Should work over and over now.
   * Added "Component Base" support for those of us who don't set our component axes.  This uses the bottom center of the component.
   * Added much more reliable FaceMe component support (I hope).
   * Added Rotation capability.  Uniform and random with user specified min/max values.
2.5 2009-12-22
   * Fixed scaling so that Face Me components don't freak out like they have been doing since 7.1 was introduced.  Big Thanks to Edson and Thom for helping figure out what was breaking.
   * Cleaned up some code - a big chunk thanks to a tip from TIG!
=end

require 'sketchup.rb'

module Clf_scale_multiple
	def Clf_scale_multiple.uniform
		ents = Sketchup.active_model.selection
		face_me = 0
		if ents.empty?
			results = UI.messagebox "Please select groups and components before starting this plugin"
			Sketchup.send_action "selectSelectionTool:"
			return
		end
		results = UI.inputbox ['Scale about', 'Scale Factor', 'Rotation'], ['Axis', '1', '0'], ['Axis|Comp. Base|Center|World|'], 'Uniformly Scale and Rotate Multiple'
		scale = results[1].to_f
		degree = results[2].to_f
		a = Sketchup.version
		b = a[0,1].to_i
		if b > 6
			Sketchup.active_model.start_operation("Scale Multiple", true)
		else
			Sketchup.active_model.start_operation("Scale Multiple")
		end
		if results[0] == "World"
			point = [0,0,0]	
		end
		ents.each do |e|
			if (e.typename == "Group") or (e.typename == "ComponentInstance")
				if e.typename == "ComponentInstance"
					behavior = e.definition.behavior
					if behavior.always_face_camera?
						face_me = true
						e.definition.behavior.always_face_camera= false
					else
						face_me = false
					end
				else
					face_me = false
				end
				if results[0] == "Center"
					point = e.bounds.center
				end
				if results[0] == "Axis"
					point = e.transformation.origin
				end
				if results[0] == "Comp. Base"
					bbox = e.bounds
					center_point = bbox.center
					point = center_point
					halfheight = (bbox.depth / 2 )
					point[2] -= halfheight
				end
				
				e.definition.behavior.always_face_camera= true if face_me
				#t = Geom::Transformation.scaling point, scale
				#This does transformations so that 7.1 will like it.  It moves the comp to the origin, 
				#then transforms using the matrix, then puts it back in place
				t_origin = Geom::Transformation.new([1,0,0,0,0,1,0,0,0,0,1,0,-point[0],-point[1],-point[2],1])
				e.transform! t_origin

				ts1 = Geom::Transformation.new([scale,0,0,0,0,scale,0,0,0,0,scale,0,0,0,0,1])
				e.transform! ts1
				 
				t_origin = Geom::Transformation.new([1,0,0,0,0,1,0,0,0,0,1,0,point[0],point[1],point[2],1])
				e.transform! t_origin
				
				#e.transform! t
				Clf_scale_multiple.rotate e, point, degree
			end
		end
		Sketchup.active_model.commit_operation
	end
	
	
	
	
	def Clf_scale_multiple.rotate object, point, degree
		vector = object.transformation.zaxis
		t = Geom::Transformation.rotation point, vector, degree.degrees
		object.transform! t
	end
	
	
	
	
	
	def Clf_scale_multiple.random
		ents = Sketchup.active_model.selection
		face_me = 0
		if ents.empty?
			UI.messagebox "Please select groups and components before starting this plugin"
			Sketchup.send_action "selectSelectionTool:"
			return
		end
		retrymenu = 1
		while retrymenu == 1
			results = UI.inputbox ['Scale about', 'Min Scale Factor  ', 'Max Scale Factor  ', 'Min Rotation', 'Max Rotation'], ['Axis', '0.5', '1.5', '0', '0'], ['Axis|Comp. Base|Center|World'], 'Randomly Scale and Rotate Multiple'
			if results[1] > results[2]
				retrymenu = 1
				UI.messagebox "Please set the minumum scale value to be smaller than the maximum scale value."
			else
				retrymenu = 0
			end
			
			if results[3].to_f > results[4].to_f
				retrymenu = 1
				UI.messagebox "Please set the minumum rotation value to be smaller than the maximum rotation value."
			else
				retrymenu = 0
			end
		end
		minr = results[3].to_f
		maxr = results[4].to_f
		ranger = maxr - minr
		min = results[1].to_f * 1000
		max = results[2].to_f * 1000
		range = max - min
		a = Sketchup.version
		b = a[0,1].to_i
		if b > 6
			Sketchup.active_model.start_operation("Scale Multiple", true)
		else
			Sketchup.active_model.start_operation("Scale Multiple")
		end
		if results[0] == "World"
			point = [0,0,0]
		end
		ents.each do |e|
			if (e.typename == "Group") or (e.typename == "ComponentInstance")
				scale = (rand(range) + min ) / 1000.0
				if e.typename == "ComponentInstance"
					behavior = e.definition.behavior
					if behavior.always_face_camera?
						face_me = true
						e.definition.behavior.always_face_camera= false
					else
						face_me = false
					end
				else
					face_me = false
				end
				if results[0] == "Center"
					point = e.bounds.center
				end
				if results[0] == "Axis"
					point = e.transformation.origin
				end
				if results[0] == "Comp. Base"
					bbox = e.bounds
					center_point = bbox.center
					point = center_point
					halfheight = (bbox.depth / 2 )
					point[2] -= halfheight
				end
				e.definition.behavior.always_face_camera= true if face_me

				#This does transformations so that 7.1 will like it.  It moves the comp to the origin, 
				#then transforms using the matrix, then puts it back in place
				t_origin = Geom::Transformation.new([1,0,0,0,0,1,0,0,0,0,1,0,-point[0],-point[1],-point[2],1])
				e.transform! t_origin

				ts1 = Geom::Transformation.new([scale,0,0,0,0,scale,0,0,0,0,scale,0,0,0,0,1])
				e.transform! ts1
				 
				t_origin = Geom::Transformation.new([1,0,0,0,0,1,0,0,0,0,1,0,point[0],point[1],point[2],1])
				e.transform! t_origin
				
				degree = (rand(ranger) + minr )
				Clf_scale_multiple.rotate e, point, degree
			end
		end
		Sketchup.active_model.commit_operation
	end
end # Clf_scale_multiple Module

if( $submenu )
	organizer = $submenu 
	if !file_loaded?(__FILE__) then
		scale_multiple = organizer.add_submenu("Scale and Rotate Multiple")
		scale_multiple.add_item("Scale.Rotate All the Same") { Clf_scale_multiple.uniform }
		scale_multiple.add_item("Scale.Rotate All Randomly") { Clf_scale_multiple.random }
	end
else
	if !file_loaded?('clf_menu_loader') then
		$clf_tools_menu = UI.menu("Plugins").add_submenu("Chris Fullmer Tools")
	end
	if !file_loaded?(__FILE__) then
		scale_multiple = $clf_tools_menu.add_submenu("Scale and Rotate Multiple")
		scale_multiple.add_item("Scale and Rotate All the Same") { Clf_scale_multiple.uniform }
		scale_multiple.add_item("Scale and Rotate All Randomly") { Clf_scale_multiple.random }
	end
end

file_loaded('clf_menu_loader')
file_loaded(__FILE__)