
# Name:           line2cyl
# Author :        Didier Bur
# Description:    Create cylinders with selected edges
# Usage:          select lines, enter diameter and precision
#
# Type:           Tool
# Date:           2005,13,01
# Revised:        2005,17,01: add the arcs and circles followme's

require 'sketchup.rb'



class ProgressBar

   @@err_total_notnumeric = "ProgressBar: Total must be a positive integer."
   @@err_count_notnumeric = "ProgressBar: Iteration Count must be numeric"
   @@end_time             = "End @ "
   @@progresschar = ">" ;
   @@initial_block = "-" * 50     # Default progress bar line sequence.

   def initialize(total,phase=nil)
      if (!total.integer? or total < 0)
         raise(ArgumentError,@@err_total_notnumeric)
         return ;
      end ;
      @total = total.to_i ;
      @phase = phase ;
      @firsttime = true ;
   end ;

   def update(iteration)
      if !iteration.integer?
         raise(ArgumentError,@@err_count_notnumeric)
         return ;
      end ;
      iteration = [iteration.abs,@total].min  # make sure we don't exceed the total count or have a value < 0
      pct = [1,(iteration*100)/@total].max    # Calculate percentage complete.
      # round up to 1% if anything less than 1%.
      end_time = "?" ;
      if @firsttime then ;
         # Get the current time of day.
         # set up an elapsed timer so we can calculate expected end time.
         @time1 = Time.now ;     # Get current time of day.
         @firsttime = false ;    # turn off switch
      else
         # divide the elapsed time by the pct complete, then multiple that by 100, then add that to the
         # start time, and that is the expected end time.
         end_time = Time.at(((((Time.now-@time1).to_f)/pct)*100 + @time1.to_f).to_i).strftime("%H:%M:%S")
      end ;
      pct_pos = [pct/2,1].max ;
      current_block = @@initial_block[0,pct_pos-1] << @@progresschar << @@initial_block[pct_pos,@@initial_block.length]
      #Sketchup.set_status_text(current_block << "   " << (pct.to_s) << "%. #{@@end_time} " << end_time << " #{@phase}")
      Sketchup.set_status_text(current_block << "   " << (pct.to_s) << "% #{@phase}")
   end ;

end ; # class ProgressBar ;

def line2cyl
   model = Sketchup.active_model
   m_ents = model.active_entities
   materials=model.materials
   ss = model.selection
   ignored = 0
   ss_edges=[]
   ss_alone = []
   ss_curves = []
   ss_connected = []
   trunk=[]
   $l2c_diam=10.cm if not $l2c_diam
   $l2c_segs=10 if not $l2c_segs
   $l2c_opt="Yes" if not $l2c_opt
   $l2c_mat="Default" if not $l2c_mat
   $l2c_grp1="Yes" if not $l2c_grp1
   $l2c_grp2="No" if not $l2c_grp2

   Sketchup.set_status_text("")

   model.start_operation "Lines -> Cylinders"
   # Selection error checking
   if ss.empty?
      UI.messagebox("No selection.")
      return nil
   end
   # Dialog
   list2=materials.collect { |m| m.name }
   if list2.length==0
      list2=["Default"]
   end
   drops=[["Yes","No"].join("|"),["Yes","No"].join("|"),["Yes","No"].join("|"),list2.join("|")]
   prompts = ["Follow me on curves: ","Create group: ","Each tube is a group: ","Material: ","Diameter: ", "Precision:  "]
   values = [$l2c_opt,$l2c_grp1,$l2c_grp2,$l2c_mat,$l2c_diam,$l2c_segs]
   results = inputbox prompts, values,drops, "Parameters"
   return if not results

   $l2c_opt = results[0]
   $l2c_grp1 = results[1]
   $l2c_grp2 = results[2]
   $l2c_mat = results[3]
   $l2c_diam = results[4]
   $l2c_segs = results[5]

   # Create group if needed
   if $l2c_grp1=="Yes"
      grp=m_ents.add_group
      ents=grp.entities
   else
      ents=m_ents
   end
   # Dispatch selection
   @pb = ProgressBar.new(ss.length,"Parsing selection...")
   pbcount=0
   #Sketchup.set_status_text("Parsing selection...")
   ss.each do |x|
      ss_edges.push(x) if x.typename=="Edge"
      pbcount+=1
      @pb.update(pbcount)
   end
   ignored=ss.length-ss_edges.length

   if $l2c_opt=="Yes"
      #Sketchup.set_status_text("Dispatching edges...")
      pbcount=0
      @pb = ProgressBar.new(ss_edges.length,"Dispatching edges...")
      ss_edges.each do |e|
         if e.curve
            ss_curves.push(e.curve)
         else
            ss_alone.push(e)
         end
         #if e.all_connected.length>1 and not e.curve
         #ss_connected.push(e.all_connected)
         #end
         #if e.all_connected.length==1
         #ss_alone.push(e)
         #end
         pbcount+=1
         @pb.update(pbcount)
      end
      ss_curves.uniq!
      #ss_connected.uniq!
      ss_alone.uniq!

      #Sketchup.set_status_text("Generating cylinders...")
      pbcount=0
      @pb = ProgressBar.new((ss_curves.length+ss_alone.length),"Generating tubes...")
      # Pushpull the standalone edges in their own group
      ss_alone.each do |e|
         vec = e.line[1]
         tube_group=ents.add_group
         tube_ents=tube_group.entities
         circle = tube_ents.add_circle(e.vertices[0], vec, ($l2c_diam / 2.0), $l2c_segs)
         if circle
            base = tube_ents.add_face(circle)
            base.material=$l2c_mat if $l2c_mat!="Default"
            if base
               base.reverse! if base.normal != vec
               base.pushpull e.length
            end
            # Store tube group in a trunk
            trunk.push(tube_group)
         end
         pbcount+=1
         @pb.update(pbcount)
      end

      # Followme's on curves
      ss_curves.each do |c|
         edge1 = c.first_edge
         vec = edge1.line[1]
         ext1 = edge1.start.position
         all_edges = c.edges
         tube_group=ents.add_group
         tube_ents=tube_group.entities
         circle = tube_ents.add_circle(ext1, vec, ($l2c_diam / 2.0), $l2c_segs)
         if circle
            base = tube_ents.add_face(circle)
            base.material=$l2c_mat if $l2c_mat!="Default"
            if base
               base.reverse!
               base.followme all_edges
            end
            # Store tube group in a trunk
            trunk.push(tube_group)
         end
         pbcount+=1
         @pb.update(pbcount)
      end

      ss_connected.each do |conn_e|
         # find first edge
         ext_edges=[]
         conn_e.each { |e| ext_edges.push(e) if e.start.edges.length==1;
         ext_edges.push(e) if e.end.edges.length==1 }

         vec = ext_edges[0].line[1]
         tube_group=ents.add_group
         tube_ents=tube_group.entities
         circle = tube_ents.add_circle(ext1, vec, ($l2c_diam / 2.0), $l2c_segs)
         if circle
            base = tube_ents.add_face(circle)
            base.material=$l2c_mat if $l2c_mat!="Default"
            if base
               #base.reverse!
               base.followme conn_e
            end
            # Store tube group in a trunk
            trunk.push(tube_group)
         end
      end
      Sketchup.set_status_text("")
   end

   if $l2c_opt=="No"
      pbcount=0
      @pb = ProgressBar.new(ss_edges.length,"Generating tubes...")
      # Pushpull the standalone edges in their own group
      ss_edges.each do |e|
         vec = e.line[1]
         tube_group=ents.add_group
         tube_ents=tube_group.entities
         circle = tube_ents.add_circle(e.start.position, vec, ($l2c_diam / 2.0), $l2c_segs)
         if circle
            base = tube_ents.add_face(circle)
            if base
               base.material=$l2c_mat if $l2c_mat!="Default"
               base.reverse! if base.normal != vec
               base.pushpull e.length
            end
            # Store tube group in a trunk
            trunk.push(tube_group)
         end
         pbcount+=1
         @pb.update(pbcount)
      end
   end

   Sketchup.set_status_text("Displaying tubes, please wait...")
   # Explode all tube groups
   if $l2c_grp2=="No"
      trunk.each { |tube| tube.explode if tube.valid? }
   end
   UI.messagebox(ss_edges.length.to_s+" tubes created,\n"+ignored.to_s+" objects were not lines and have been ignored.")
   Sketchup.set_status_text("")
   model.commit_operation
end #of def



#~ if( not file_loaded?("line2cyl.rb") )
#~ add_separator_to_menu("Plugins")
#~ UI.menu("Tools").add_item("Convert arcs,circles,curves,lines to cylinders") { line2cyl }
#~ end
#~ file_loaded("line2cyl.rb")
