require 'sketchup'
require 'progressbar'
Sketchup.require 'protrude/corner_vertices.rbs'

def edgeMidpoint(edge)
  if edge.typename == "Edge"
    s = edge.start.position
    e = edge.end.position
    return midPoint(s, e)
  else
    return nil
  end
end

def midPoint(p1, p2)
  Geom::Point3d.new( (p1.x + p2.x / 2), (p1.y + p2.y) / 2, (p1.z + p2.z) / 2)
end

def arrayMidpoint(a)
  midPoint(a[0], a[1])
end

def faceCenterPoint(face)
  oloop = face.outer_loop
  iloop = face.loops - [oloop]
  if iloop.length > 0
    verts = iloop[0].vertices
  else
    verts = oloop.vertices
  end
  verts = SK::corner_vertices(face)
  #verts.each {|v| Sketchup.active_model.entities.add_cpoint v.position}
  #verts = face.vertices
  #verts.push verts[0]
  x = verts.collect{|v| v.position.x}.inject{|s, n| s += n}
  y = verts.collect{|v| v.position.y}.inject{|s, n| s += n}
  z = verts.collect{|v| v.position.z}.inject{|s, n| s += n}
  len = verts.length
  Geom::Point3d.new( [x / len, y / len, z / len] )
end

def random_face_divide(min, max)
  model = Sketchup.active_model
  selection = model.selection
  face = selection[0]
  $depth = 1
  min = Integer(min)
  max = Integer(max)
  max = 50
  faces = selection.find_all{ |e| e.typename == "Face" }
  pb = ProgressBar.new(faces.length)
  pbc = 0
  faces.each do |face|
    pb.update(pbc+=1)
    r = Integer( rand(max - min) + min )
    #r = Integer( rand(min) )
    verts = SK::corner_vertices(face)
    next if verts.length != 4
    #p verts
    verts.each_with_index { |v, i|
      #model.entities.add_text i.to_s, v.position
    }
    pts = []
    verts.each { |v| pts.push v.position }
    vecs = [pts[0].vector_to(pts[1]), pts[0].vector_to(pts[2]), pts[0].vector_to(pts[3]) ]
    lens = []
    vecs.each {|v| lens.push v.length}
    imin = lens.index lens.min
    sp1 = pts[0]
    sp2 = pts[0] + vecs[imin]
    #model.entities.add_text "sp1", sp1
    #model.entities.add_text "sp2", sp2
    # delete out the diagonal (longest), which leaves the 2 sides
    lens.delete_at imin
    vecs.delete_at imin
    imin = lens.index lens.min # new min is long side
    vec = vecs[imin]
    vec.length = vec.length * r / 100.0
    sp3 = sp1 + vec
    sp4 = sp2 + vec
    # Text Geom::intersect_line_line
    #tp1 = Geom::intersect_line_line( [sp3,  sp4], [sp1, vecs[imin]] )
    #model.entities.add_text "tp1",  tp1
    #tp2 = Geom::intersect_line_line( [sp3,  sp4], [sp2, vecs[imin]] )
    #model.entities.add_cpoint tp2
    #model.entities.add_text "sp3", sp3
    #model.entities.add_text "sp4", sp4
    edge = model.entities.add_edges sp3, sp4
    newfaces = edge[0].faces
    selection.add newfaces
    selection.remove selection.find_all{|e| e.is_a? Sketchup::Edge}
  end # faces.each
  #UI.beep
end

def qdivide
  faces = Sketchup.active_model.selection.find_all{|e| e.typename=="Face"}
  Sketchup.active_model.start_operation "qdivfaces"
  faces.each do |face|
    verts = SK::corner_vertices face
    ctr = face.bounds.center
    n = rand(2) + 3
    edges = []
    for i in -1..verts.size-2
      edges.push([verts[i].position, verts[i+1].position])
    end
    centers = []
    edges.each do |e|
      midpoint = [ ((e[0].x + e[1].x) / 2), ((e[0].y + e[1].y) / 2), ((e[0].z + e[1].z) / 2) ]
      centers << midpoint
      #center << Centroid.go(face)
    end
    centers = centers.sort_by { rand }
    n.times do |i|
      edge = Sketchup.active_model.active_entities.add_edges ctr, centers[i]
      Sketchup.active_model.selection.add edge[0].faces
    end
  end
  Sketchup.active_model.commit_operation
end

def qdivfaces
  faces = Sketchup.active_model.selection.select{|e| e.typename=="Face"}
  Sketchup.active_model.start_operation "qdivfaces"
  faces.each do |face|
    verts = SK::corner_vertices face
    ctr = face.bounds.center
    ctr = faceCenterPoint(face)
    n = rand(2) + 3
    edges = []
    for i in -1..verts.size-2
      edges.push([verts[i].position, verts[i+1].position])
    end
    centers = []
    edges.each do |e|
      midpoint = [ ((e[0].x + e[1].x) / 2), ((e[0].y + e[1].y) / 2), ((e[0].z + e[1].z) / 2) ]
      centers << midpoint
      #center << Centroid.go(face)
    end
    centers = centers.sort_by { rand }
    n.times do |i|
      edge = Sketchup.active_model.active_entities.add_edges ctr, centers[i]
      Sketchup.active_model.selection.add edge[0].faces
    end
  end
  Sketchup.active_model.commit_operation
end


#unless file_loaded? __FILE__
#UI.menu.add_item(File.basename(__FILE__)) { random_face_divide }
#file_loaded __FILE__
#end

