# SLightly modifies versionof RickW's offset.rb
#
# Copyright 2004, Rick Wilson 
#
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted, provided that the above
# copyright notice appear in all copies.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
# Name :          offset.rb 2.0
# Description :   Offset edges of a selected face (new method for class Sketchup::Face)
# Author :        Rick Wilson
# Usage :         1.	Intended for developers as a method to call from 
#				within a script.  Add a "require 'offset.rb' line
#				right after the "require 'sketchup.rb' line.  Distribute
#				with your script since not everyone will have this
#				already.  Returns the face created by the offset.
#			2.	Added method: ArcCurve.offset(dist) -- if dist is (+), offsets
#				outside the curve (larger radius); if dist is (-), offsets
#				inside the curve (smaller radius).
#			3.	Added method: Curve.offset(dist) -- if dist is (+), offsets
#				to the right of the curve (relative to the first segment direction
#				and plane); if dist is (-), offsets to the left of the curve.
# Date :          7.Sept.2004
# Type :          Methods
# History:
#			2.0 (12.Aug.2005) - added offset methods for ArcCurve and Curve objects
#			1.0 (7.Sept.2004) - first version
# 

class Sketchup::Face
  def offset(dist)
    #		return nil if dist==0
    pi=Math::PI
    if (not ((dist.class==Fixnum || dist.class==Float || dist.class==Length) && dist!=0))
      return nil
    end
    verts=self.outer_loop.vertices
    pts=[]
    0.upto(verts.length-1) do |a|
      vec1=(verts[a].position-verts[a-(verts.length-1)].position).normalize
      vec2=(verts[a].position-verts[a-1].position).normalize
      vec3=(vec1+vec2).normalize
      if vec3.valid?
	ang=vec1.angle_between(vec2)/2
	ang=pi/2 if vec1.parallel?(vec2)
	vec3.length=dist/Math::sin(ang)
	t=Geom::Transformation.new(vec3)
	if pts.length > 0
	  if not (vec2.parallel?(pts.last.vector_to(verts[a].position.transform(t))))
	    t=Geom::Transformation.new(vec3.reverse)
	  end
	end
	pts.push(verts[a].position.transform(t))
      end
    end
    #Sketchup.active_model.active_entities.add_face(pts)
    #Sketchup.active_model.selection.toggle self
    face = self.parent.entities.add_face(pts)
    #Sketchup.active_model.selection.toggle face
    #p pts
    #-1.upto(pts.length-2) { |i|
      #p i
      #p pts[i]
      #self.parent.entities.add_edges pts[i], pts[i+1]
    #}
    face
  end
end

class Array
  def offsetPoints(dist)
    #		return nil if dist==0
    pi=Math::PI
    if (not ((dist.class==Fixnum || dist.class==Float || dist.class==Length) && dist!=0))
      return nil
    end
    verts=[]
    self.each do |vert|
      case vert.class
      when Sketchup::Vertex
	verts<<vert.position
      when Array
	if vert.length==3
	  verts<<Geom::Point3d.new(vert)
	else
	  verts<<Geom::Point3d.new(vert[0..2])
	end
      when Geom::Point3d
	verts<<vert
      end
    end
    verts.uniq!

    pts=[]
    0.upto(verts.length-1) do |a|
      if verts[a-(verts.length-1)].class==Sketchup::Vertex
	pt2=verts[a-(verts.length-1)].position
      elsif verts[a-(verts.length-1)].class==Geom::Point3d
	pt2=verts[a-(verts.length-1)]
      elsif verts[a-(verts.length-1)].class==Array && verts[a-(verts.length-1)].length==3
	pt2=Geom::Point3d.new(verts[a-(verts.length-1)])
      else
	return nil
      end
      if verts[a-1].class==Sketchup::Vertex
	pt1=verts[a-1].position
      elsif verts[a-1].class==Geom::Point3d
	pt1=verts[a-1]
      elsif verts[a-1].class==Array && verts[a-1].length==3
	pt1=Geom::Point3d.new(verts[a-1])
      else
	return nil
      end
      if verts[a].class==Sketchup::Vertex
	pt3=verts[a].position
      elsif verts[a].class==Geom::Point3d
	pt3=verts[a]
      elsif verts[a].class==Array && verts[a-1].length==3
	pt3=Geom::Point3d.new(verts[a])
      else
	return nil
      end
      vec1=(pt3-pt2).normalize
      vec2=(pt3-pt1).normalize
      vec3=(vec1+vec2).normalize
      if vec3.valid?
	ang=vec1.angle_between(vec2)/2
	ang=pi/2 if vec1.parallel?(vec2)
	vec3.length=dist/Math::sin(ang)
	t=Geom::Transformation.new(vec3)
	if pts.length > 0
	  if verts[a].class==Sketchup::Vertex
	    if not (vec2.parallel?(pts.last.vector_to(verts[a].position.transform(t))))
	      t=Geom::Transformation.new(vec3.reverse)
	    end
	  elsif verts[a].class==Geom::Point3d
	    if not (vec2.parallel?(pts.last.vector_to(verts[a].transform(t))))
	      t=Geom::Transformation.new(vec3.reverse)
	    end
	  elsif verts[a].class==Array && verts[a].length==3
	    if not (vec2.parallel?(pts.last.vector_to(Geom::Point3d.new(verts[a]).transform(t))))
	      t=Geom::Transformation.new(vec3.reverse)
	    end
	  end
	end
	if verts[a].class==Sketchup::Vertex
	  pts.push(verts[a].position.transform(t))
	elsif verts[a].class==Geom::Point3d
	  pts.push(verts[a].transform(t))
	elsif verts[a].class==Array && verts[a].length==3
	  pts.push(Geom::Point3d.new(verts[a].transform(t)))
	end
      end
    end
    pts
  end
end

class Sketchup::ArcCurve
  def offset(dist)
    return nil if dist==0 || (not (dist.class==Float || dist.class==Fixnum || dist.class==Length))
    radius=self.radius+dist.to_f
    #Sketchup.active_model.active_entities.add_arc self.center, self.xaxis, self.normal, radius, self.start_angle, self.end_angle, self.count_edges
    c=Sketchup.active_model.active_entities.add_arc self.center, self.xaxis, self.normal, radius, self.start_angle, self.end_angle, self.count_edges
    c.first.curve
  end
end

class Sketchup::Curve
  def offset(dist)
    return nil if self.count_edges<2
    pi=Math::PI
    if (not ((dist.class==Fixnum || dist.class==Float || dist.class==Length) && dist!=0))
      return nil
    end
    verts=self.vertices
    pts=[]
    0.upto(verts.length-1) do |a|
      if a==0 #special case for start vertex
	model=self.model
	model.start_operation "offset"
	gp=model.active_entities.add_group
	gpents=gp.entities
	face=gpents.add_face(verts[0].position,verts[1].position,verts[2].position)
	zaxis=face.normal
	v=self.edges[0].line[1]
	f=dist/dist.abs
	t=Geom::Transformation.rotation(verts[0].position,zaxis,(pi/2)*f)
	vec3=v.transform(t)
	vec3.length=dist.abs
	pts.push(verts[0].position.transform(vec3))
	gp.erase!
	model.commit_operation
      elsif a==(verts.length-1) #special case for end vertex
	model=self.model
	model.start_operation "offset"
	gp=model.active_entities.add_group
	gpents=gp.entities
	face=gpents.add_face(verts[a].position,verts[a-1].position,verts[a-2].position)
	zaxis=face.normal
	v=self.edges[a-1].line[1]
	f=dist/dist.abs
	t=Geom::Transformation.rotation(verts[a].position,zaxis,(pi/2)*f)
	vec3=v.transform(t)
	vec3.length=dist.abs
	pts.push(verts[a].position.transform(vec3))
	gp.erase!
	model.commit_operation
      else
	vec1=(verts[a].position-verts[a-(verts.length-1)].position).normalize
	vec2=(verts[a].position-verts[a-1].position).normalize
	vec3=(vec1+vec2).normalize
	if vec3.valid?
	  ang=vec1.angle_between(vec2)/2
	  ang=pi/2 if vec1.parallel?(vec2)
	  vec3.length=dist/Math::sin(ang)
	  t=Geom::Transformation.new(vec3)
	  if pts.length > 0
	    if not (vec2.parallel?(pts.last.vector_to(verts[a].position.transform(t))))
	      t=Geom::Transformation.new(vec3.reverse)
	    end
	  end
	  pts.push(verts[a].position.transform(t))
	end
      end
    end
    #Sketchup.active_model.active_entities.add_curve(pts)
    c=Sketchup.active_model.active_entities.add_curve(pts)
    c.first.curve
  end

end

# This is the end
__END__

def add_rect(face)
  #v = corner_vertices(face)
  c = face.bounds.center
  pts = v.map{|e| e.position}
  np = []
  pts.each {|pt|
    v = c - pt
    v.length = v.length * rand
    np.push (pt + v)
  }
  np.each {|pt| Sketchup.active_model.entities.add_cpoint pt }
  f = Sketchup.active_model.active_entities.add_face np
  f.pushpull(f.area / 60, true)
end

def self.protrude
  @opts = { 'taper' => 0.8, 'height' => 6, 'type' => "Random" }  unless @opts

  r = UI.inputbox(
    ["Taper (0.1 - 1)"          ,"Height"         , "Type" ],
    [@opts['taper']   , @opts['height'] , @opts['type']],
    [nil              , nil             , "Uniform|Random"]
  )
  return unless r

  @opts['taper'] = r[0].to_f
  if @opts['taper'] < 0.1
    @opts['taper'] = 0.1
  end

  @opts['height'] = r[1].to_l
  @opts['type'] = r[2]

  faces = Sketchup.active_model.selection.select{|e| e.typename=="Face" }
  ss=faces.length
  #pb = ProgressBar.new(ss, "Protrude")
  #i = 0
  faces.each { |f|
    protrude(f)
    #pb.update(i)
    #i += 1
  }
end


