# open a fold: zo on fold
# create a fold: zf (visual mode)
#require "dev/debug.rb"
require 'sketchup'

pf = Sketchup.find_support_file("Plugins")
if !File.exists?(File.join(pf, "progressbar.rb"))
  UI.messagebox("Please install progressbar.rb from smustard.com")
else
  require 'progressbar'
end
Sketchup.require 'protrude/corner_vertices.rbs'
Sketchup.require 'protrude/divide_faces.rbs'
Sketchup.require 'protrude/offset_mod.rbs'
#Sketchup.require 'protrude/randomselect.rbs'
Sketchup.require 'protrude/selection_manager.rbs'
Sketchup.require 'protrude/centroid.rbs'

def round_to(x, y=Sketchup.active_model.options['UnitsOptions']['LengthPrecision'])
  (x * 10**y).round.to_f / 10**y
end

class ProtrudeDialog < UI::WebDialog
  @@instance = nil
  def initialize
    @tops = []
    @sm = SelectionManager.new
    super "Protrude", true, "Protrude"
    #loadGreebles
    set_file( File.join(Sketchup.find_support_file("Plugins") , "protrude/ui.html"))
    add_action_callback("pform") { |d, a| pform(d, a) }
    add_action_callback("set_opts") { |d, a| set_opts(d, a) }
    #add_action_callback("protrude") { |d, a| protrude(d, a) }
    #add_action_callback("qdivide") { qdivfaces; setInner("face_count", @sm.active_faces.length) }
    #add_action_callback("undo") { my_undo }
    add_action_callback("redo") { my_redo }
    add_action_callback("do_over") { |d, a| do_over() }
    add_action_callback("keep") { |d, a| keep() }
    #add_action_callback("reduce") { |d, a| reduce() }
    add_action_callback("addGreebles") { |d, a| addGreebles() }
    add_action_callback("usePreset") { |d, a| usePreset() }
    #add_action_callback("setReasonableValues") { |d, a| setReasonableValues() }
    add_action_callback("sm_inspect") { puts @sm.inspect }
    add_action_callback("ids") { p @sm.active.map{ |e| e.entityID } }
    #add_action_callback("tops") { select_tops }
    #add_action_callback("guesstops") { guess_tops }
    #add_action_callback("sides") { select_sides }
    add_action_callback("centroid") { Sketchup.active_model.entities.add_cpoint Centroid.go(@sm.active[0]) }
    add_action_callback("fcp") { Sketchup.active_model.entities.add_cpoint faceCenterPoint(@sm.active[0]) }
    add_action_callback("slider") {|d, a| slider(a) }
    add_action_callback("smpush") {|d, a| slider(a) }
    add_action_callback("smpop") {|d, a| slider(a) }
    add_action_callback("smprev") {|d, a| slider(a) }
    add_action_callback("smmnext") {|d, a| slider(a) }

    # skp callback
    add_action_callback("cb") do |d, a|
      a.gsub!(/\(nil\)/, "()")
      puts "rcvd callback: #{a.inspect}"
      eval(a)
    end 

    set_on_close { onClose }
    @opts = {}
    show { }
  end # initialize

  def sel_store
    @sm.set
  end

  def sel_restore
    @sm.restore
  end

  def smpush(v=nil)
    puts "smpush:#{v.inspect}"
    @sm.push
  end

  def smcurrent
    puts @sm.current
  end

  def smset
    @sm.setSelection
  end

  def smclear
    @sm.clear
  end

  def smprev
    @sm.prev
  end
  def smnext
    @sm.next
  end
  def smins
    p @sm
  end
  def sm_shuffle(s=50)
    @sm.shuffle(s)
  end


  def slider(a)
    #puts "slider:#{a.inspect}"
    a = Integer(a)
    @sm.reduce(a)
    #setInner("face_count", @sm.active_faces.length)
  end

  def my_undo
    Sketchup.send_action "editUndo:"
    @sm.undo 
  end # my_undo

  def do_over
    #my_undo
    Sketchup.send_action "editUndo:"
    @sm.undo 
    UI.start_timer(1) { protrude }
  end

  def my_redo
    #st = Sketchup.send_action "editRedo:"
    #UI.start_timer(0.1) { @sm.next }
  end


  def divideFaces
    Sketchup.active_model.start_operation "Divide"
    @sm.save
    min = @opts['dmin'][0]
    max = @opts['dmax'][0]
    random_face_divide(min, max)
    @sm.save2
    #setInner("face_count", @sm.active_faces.length)
    #setById('slider', 100)
    #execute_script('document.getElementById("slider").click();')
    Sketchup.active_model.commit_operation
  end

  def onClose 
    print "Closing."
    execute_script("setOpts();")
    %w( dmin dmax omin omax hmin hmax tmin tmax ).each do |o|
      @opts[o] && Sketchup.write_default("JFProtrudeDialog", o, @opts[o][0])
    end
  end

  def self.close
    @@instance && @@instance.close
  end

  def self.toggle
    if @@instance == nil
      @@instance = self.new
      @@instance.show { }
    else
      if @@instance.visible?
	@@instance.close
      else
	@@instance.show { }
      end
    end
  end


  def set_opts(d, a)
    #puts "set_opts:#{a.inspect}"
    opts = {}
    return if a == "undefined"
    a.split(",").each do |opt|
      k, v = opt.split("=")
      opts[k] ||= []
      opts[k].push(v)
    end
    p opts
    @opts = opts
  end


  def protrude
    faces = @sm.active_faces
    @sm.save
    pb = ProgressBar.new faces.length
    pbc = 0
    #grp = Sketchup.active_model.active_entities.add_group face
    #@group.erase! if @group && @group.valid?
    @tops = []
    @sides = []
    ofaces = []
    @bots = []
    bases = []
    Sketchup.active_model.start_operation "Protrude"
    faces.each do |face|
      pb.update(pbc+=1)
      bface = doOffset(face)
      bface = face if bface.nil?
      bases << bface
      if calcHeight > 0
	top, sides = doProtrude(bface)
	#puts "top:#{top.inspect}"
	#puts "sides:#{sides.inspect}"
	@tops << top
	@sides += sides
	doScaling top if calcTaper > 0
	#SketchyBevel.bevel([top] | sides, 12, true)
      end
    end
    #@sm.save
    #@sm.setSelection
    if !@opts['keep_bottoms']
      bases.map {|e| e.erase! unless calcHeight == 0}
    end
    #@tops.reject!{ |e| !e.valid?}
    #setSelection @tops #if @opts['tops']
    select_tops
    #@sm.save
    #execute_script("reduce();")
    #Sketchup.active_model.active_view.zoom_extents
    Sketchup.active_model.commit_operation
    #puts "@sides:#{@sides.inspect}"
  end

  def doScaling(face)
    tmin = @opts['tmin'][0].to_f if @opts['tmin']
    taper = tmin
    if @opts['tmax']
      tmax = @opts['tmax'][0].to_f if @opts['tmax']
      taper = ((tmax - tmin + 1) * rand) + tmin
    end
    taper = 99 if taper >= 100
    #puts "taper:#{ taper.inspect }"

    tr = Geom::Transformation.scaling(face.bounds.center, 1.0 - taper/100)
    face.parent.entities.transform_entities tr, face unless taper == 0
    face
  end	
  def myrand(min, max)
    (rand * (max -  min) + min)
  end


  def doOffset(face)
    offset = calcOffset()
    face.offset(-calcOffset)
  end
  def calcOffset
    omin = @opts['omin'][0].to_l
    offset = omin
    if @opts['omax']
      omax = @opts['omax'][0].to_l
      offset = ((omax - omin) * rand) + omin  unless omax == 0
    end
    offset
  end


  def calcHeight
    min = @opts['hmin'][0].to_l
    if @opts['hmax']
      max = @opts['hmax'][0].to_l 
      h = ((max - min) * rand) + min
    else
      h = min
    end
    h
  end

  def calcTaper
    #p @opts['tmin']
    min = @opts['tmin'] ?  @opts['tmin'][0].to_f : 0
    #p min
    #p @opts['tmax']
    max = @opts['tmax'] ? @opts['tmax'][0].to_f : 0
    #p max
    if min > max
      max = min
    end
    t = ((max - min) * rand) + min
  end

  def doProtrude(face)
    entities = face.parent.entities
    n = face.normal
    h = calcHeight
    n.length = h
    tops = []
    bots = []
    verts = face.outer_loop.vertices
    verts.each {|v|
      b_pt = v.position
      top_pt = b_pt + n
      tops << top_pt
      bots << b_pt
    }
    sides = []
    topface = entities.add_face tops
    for i in 0..verts.length-1
      sides << entities.add_face(bots[i-1], tops[i-1], tops[i], bots[i])
    end
    [topface, sides]
  end

  def doProtrudeOld(face)
    n = face.normal
    b = face.bounds
    #min = @opts['hmin'][0].to_l
    #if @opts['hmax']
    #max = @opts['hmax'][0].to_i 
    #h = Integer(((max - min + 1) * rand) + min)
    #else
    #h = min
    #end
    h = calcHeight
    st = face.pushpull(h, true)
    #p st
    #face.pushpull(guess_height(face), true)
    #face.reverse!
  end

  def doRayTest(face)
    #fcp = faceCenterPoint(face)
    fcp =Centroid.go(face)
    Sketchup.active_model.entities.add_cpoint fcp
    rt = Sketchup.active_model.raytest([fcp, face.normal])
    p rt
    #p c = Centroid.go(face)
    #Sketchup.active_model.active_entities.add_cpoint c
    if rt.nil?
      rt = Sketchup.active_model.raytest([face.bounds.center, face.normal])
    end
    if rt
      path = rt[1]
      #p path
      @tops << path.first
      ctr = path.first.bounds.center
      #Sketchup.active_model.entities.add_cpoint ctr
      #Sketchup.active_model.selection.add path.first
      return path.first
    else
      puts "Raytest failed."
      return nil
    end # if rt
  end # doRayTest

  def offsetByPercent(face)
    verts = face.vertices
    v0 = verts.shift
    dists = verts.map { |v| v0.position.distance(v.position) }
    dists.sort!
    dists.pop
    #p offset
    dist = (dists.min) * calcOffset() / 100
    face.offset(-1.0 * dist)
  end

  def setSelection(c)
    Sketchup.active_model.selection.clear
    Sketchup.active_model.selection.add c
  end

  def keep
    c = @group.to_component
    c.erase!
    #@group.erase!
  end

  def guess_height(face)
    h = faceMinMax[1] * 0.1
  end

  def faceMinMax(face)
    verts = SK::corner_vertices face
    v0 = verts.shift
    verts.map!{ |v| v0.position.distance( v.position ) }
    [verts.min, verts.max]
  end

  def offsetFromAngle(a, h)
    offset = h * Math::tan(a.degrees)
    puts "offset from angle = #{offset}"
  end

  def shortestEdge
    #faces = Sketchup.active_model.selection.find_all{ |e| e.typename == "Face" }
    faces = @sm.active_faces
    s = nil
    faces.each do |face|
      mm = faceMinMax(face)
      s = mm[0] if s.nil?
      if mm[0] < s
	s = mm[0]
      end
    end
    s
  end

  def analyse_model
    
  end

  def reduce
    @sm.save2
    @sm.save
    @sm.reduce(75)
    #setFaceCount
  end
  def setFaceCount()
    # setInner("face_count", @sm.active_faces.length)
  end

  def setById(id, val=nil)
    #p val
    if val.nil?
      val = ""
    else
    val = val.to_l.to_s
    #val = round_to( val.to_l ).to_s
    end
    val.gsub!(/'/, "\\'")
    val.gsub!(/"/, '\"')
    val.gsub!(/~ /, '')
    #cmd = "document.getElementById('#{id}').value='#{val}';"
    #cmd = %/$('##{id}').val("#{val}");/
    cmd = %/setById('#{id}', '#{val}')/
    #p cmd
    execute_script(cmd)
  end
  def setInner(id, val)
    #cmd = "document.getElementById('#{id}').innerHTML='#{val}'"
    cmd = "setInner('#{id},#{val}')"
    p cmd
    self.execute_script(cmd)
  end

  def usePreset
    type = @opts['protrude_presets'][0]
    case type
    when "high-city"
      setById('omin', shortestEdge() * 0.1 )
      setById('omax', "")
      setById('hmin', shortestEdge() * 1 )
      setById('hmax', shortestEdge() * 20 )
      setById('tmin', 0)
      setById('tmax', '')
    when "med-city"
      setById('omin', shortestEdge() * 0.1 )
      setById('hmin', shortestEdge() * 1 )
      setById('hmax', shortestEdge() * 10 )
      setById('tmin', 0)
      setById('tmax', '')
    when "low-city"
      setById('omin', shortestEdge() * 0.1 )
      setById('hmin', shortestEdge() * 1 )
      setById('hmax', shortestEdge() * 3 )
      setById('tmin', 0)
      setById('tmax', '')
    when "low-taper"
      setById('tmin', 0)
      setById('tmax', 35)
    when "medium-taper"
      setById('tmin', 35)
      setById('tmax', 70)
    when "high-taper"
      setById('tmin', 70)
      setById('tmax', 95)
    end
  end

  def setReasonableValues
    puts "setResonableValues:"
    setById('omin', shortestEdge() * 0.1 )
    setById('omax')
    setById('hmin', shortestEdge() )
    #setById('hmax', shortestEdge() * 3)
    setById('hmax')
    #setById('tmin', 0)
    setById('tmax')
  end

  def reasonableTaper
    setById('tmin', 0)
    setById('tmax')
  end
  def reasonableHeight
    #setById('hmin', shortestEdge() )
    setById('hmin', shortestEdge()  )
    setById('hmax')
  end
  def reasonableOffset
    setById('omin', shortestEdge() * 0.1 )
    setById('omax')
  end


  def loadGreebles
    names = %w( n1.skp n2.skp n3.skp n4.skp  )
    @nurnies = []
    names.each do |name|
      path = Sketchup.find_support_file name, "Plugins/protrude/nurnies"
      tdef = nil
      tdef = Sketchup.active_model.definitions[name]
      next if tdef
      cdef = Sketchup.active_model.definitions.load(path)
      #b = cdef.behavior
      #b.is2d = true
      @nurnies << cdef
    end
  end

    def addGreebles
    Sketchup.active_model.start_operation "Add Nurnies"
    @sm.active.each do |face|
      next unless face.typename == "Face"
      area = face.area
      normal = face.normal
      center = faceCenterPoint(face)
      #p center
      fw = face.bounds.width
      fh = face.bounds.height
      fw, fh = faceMinMax(face)
      trans = Geom::Transformation.new(center, normal)
      ins = Sketchup.active_model.entities.add_instance(@nurnies[rand(@nurnies.length)], trans)
      rr = Geom::Transformation.rotation(ins.transformation.origin, Z_AXIS, [0, 90, 180, 270][rand(4)].degrees)
      ins.transform! rr
      #if ins.bounds.width > ins.bounds.height
      #  fw, fh = fh, fw
      #end
      xscale = fw * myrand(0.1, 0.5)
      #xscale = ins.bounds.width * 0.5
      #xscale = fw
      yscale = fh * myrand(0.1, 0.5)
      #yscale = ins.bounds.height * 0.5
      # can't use calcHeight - it can change
      #zscale = calcHeight * myrand(0.05, 0.1) + 0.1
      zscale = (xscale + yscale / 2.0) * myrand(0.1, 0.2)
      scale = Geom::Transformation.scaling(ins.transformation.origin, xscale, yscale, zscale)
      ins.transform! scale
      #rr = Geom::Transformation.rotation(ins.bounds.corners[1], Z_AXIS.cross(normal), Z_AXIS.angle_between(normal))
      #ins.transform! rr
      #ins.transformation *= scale
      # Rotate 90, 180, or 270
      #tr *= rr
      
      #ins.transform! rr
    end
    Sketchup.active_model.commit_operation
  end

  def select_tops
    Sketchup.active_model.selection.clear
    Sketchup.active_model.selection.add @tops
    #setInner("face_count", @sm.active_faces.length)
  end
  def sides
    Sketchup.active_model.selection.clear
    Sketchup.active_model.selection.add @sides
  end

  def guess_tops
    faces = Sketchup.active_model.active_entities.select{|e| e.typename=="Face" and e.normal == Z_AXIS}
    faces = faces.select{|f| f.vertices[0].position.z > 0}
    #faces.reject! { |f| f.normal != Z_AXIS }
    faces.reject!{|f| f.loops.length > 1}
    Sketchup.active_model.selection.clear
    Sketchup.active_model.selection.add faces
    #setInner("face_count", @sm.active_faces.length)
  end


  def bevel
    @sm.clear

  end


end # class ProtrudeDialog


